import * as service from '../scss/admin.scss';
import CustomerBookings from './components/customer-bookings.mjs';
import SettingsFormCustomizer from './components/settings-form-customizer.mjs';
import HtmlExtensionRegistry from './html/html-extension-registry.mjs';
import SettingsShortcodeGenerator from "./components/settings-shortcode-generator.mjs";

(function (_w, _d) {
    let urlParams = new URLSearchParams(window.location.search);
    let actions = {
        'wpcb_settings.php': eventsForm,
        'cb_form_settings.php': optionForm,
        'cb_shortcode_generator.php': shortCodeGenerator,
        'customer_bookings.php': customerBookingsPage,
        'wpcb_form_customize_settings.php': SettingsFormCustomizerPage,
    };

    let action = actions[urlParams.get('page')];
    if (action) {
        action();
    }

    _w.addEventListener('keydown', function (evt) {

        let combinations = [
            // combination of alt + d and for mac cmd + d
            evt.key === 'd' && (evt.altKey === true || evt.metaKey === true),

            // combination of alt + h and for mac cmd + h
            evt.key === 'h' && (evt.altKey === true || evt.metaKey === true),
        ]

        console.log(combinations);
        //any combination of key press
        if (combinations.includes(true)) {
            console.log('key pressed');
            evt.preventDefault();
            //reload window adding an extra parameter to the url
            _w.location.href = _w.location.href + '&debug_mode'
            return false;
        }
    });

    _w.addEventListener('click', function (evt) {
        if (evt.target.id.indexOf('delete-wpcb-caches') > -1) {
            evt.preventDefault();

            let answer = prompt('Are you sure you want to delete all caches? Type "yes" to confirm.', 'no');
            if (answer === 'yes') {
                DeleteTransients().then((data) => {
                    _d.querySelector('.jims-d-menu-toolbar .notifications').innerHTML = `caches deleted`;
                });
            }
            return false;
        }

        if (evt.target.id.indexOf('update-leads-record') > -1) {
            evt.preventDefault();

            UpdateLeadsRecord().then((data) => {
                _d.querySelector('.jims-d-menu-toolbar .notifications').innerHTML = `Leads Updated`;
            });

            return false;
        }
    });

    function customerBookingsPage() {
        const customerBookingsData = new CustomerBookings();

        if (window.graphData !== undefined) {
            customerBookingsData.barChart('booking-overview', graphData.success, graphData.failed);
        }
    }

    function SettingsFormCustomizerPage() {
        const customizer = _d.querySelector('#jims-form-customizer');

        if (window.jims_wpcb_variables === undefined || customizer === null) {
            throw new Error('jims_wpcb_variables.home_url is undefined');
        }

        const settingsFormCustomizer = new SettingsFormCustomizer(customizer, jims_wpcb_variables);
        settingsFormCustomizer.init();
    }

    /**
     *
     */
    function optionForm() {
        const endpoint = `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/form`;
        const admin_endpoint = `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/form_admin`;

        _d.querySelector('#jims-form').addEventListener('submit', (evt) => {
            evt.preventDefault();
            let formdata = new FormData(evt.target);
            evt.target.querySelector('input[type=submit]').classList.add('disabled');
            SaveData(formatMessage(formdata), evt.target);

            return false;
        });

        _d.querySelector('#delete-form').addEventListener('click', (evt) => {
            evt.preventDefault();
            _d.querySelector('#delete-form').classList.add('disabled');
            DeleteForm();

            return false;
        });


        // _d.querySelector('#refresh-form').addEventListener('click', (evt) => {
        //     evt.preventDefault();
        //     _d.querySelector('#refresh-form').classList.add('disabled');
        //     RefreshForm( document.querySelector('#jims-form') );
        //
        //     return false;
        // });

        async function SaveData(data, formElement) {
            const rawResponse = await fetch(endpoint, {
                method: 'POST',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(data)
            });
            const content = await rawResponse.json();
            formElement.querySelector('input[type=submit]').classList.remove('disabled');
            console.log(content);
        }

        async function DeleteForm() {
            if (typeof wpcbSecret == 'undefined') return;

            const rawResponse = await fetch(admin_endpoint, {
                method: 'POST',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({'allow': wpcbSecret})
            });
            const content = await rawResponse.json();
            _d.querySelector('#delete-form').classList.remove('disabled');
            console.log(content);
        }

        async function RefreshForm(formElement) {

            const rawResponse = await fetch(admin_endpoint, {
                method: 'GET',
                headers: {
                    'Accept': 'application/json',
                },
            });
            const content = await rawResponse.json();
            if (content.hasOwnProperty('fields') && content.fields.length > 0) {
                Object.keys(content.fields).forEach(key => {
                    if (formElement && formElement.elements[key]) {
                        let field = formElement.elements[key];
                        if (input) {
                            field.value = content.fields[key];
                        }
                    }
                });
            }

            _d.querySelector('#refresh-form').classList.remove('disabled');
            console.log(content);
        }
    }

    function eventsForm() {
        const endpoint = `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/form-events`;
        const adminEndpoint = `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/form-events-admin`;

        const form = _d.querySelector('#jims-events-form');
        const divisionId = _d.querySelector('#divisions').value;

        console.log('running events form');
        form.querySelectorAll('input[type=password]').forEach((input) => {
            //add html next to the input
            let showPassWordButton = _d.createElement('button');
            showPassWordButton.innerHTML = 'Show Hidden Content';
            showPassWordButton.type = 'button';
            showPassWordButton.classList.add('show-password');
            showPassWordButton.addEventListener('click', (evt) => {
                evt.preventDefault();
                if (input.type === 'password') {
                    input.type = 'text';
                    showPassWordButton.innerHTML = 'Hide Content';
                } else {
                    input.type = 'password';
                    showPassWordButton.innerHTML = 'Show Hidden Content';
                }
            });

            input.parentNode.appendChild(showPassWordButton);
        });

        form.addEventListener('submit', (evt) => {
            evt.preventDefault();
            let formdata = new FormData(evt.target);
            evt.target.querySelector('input[type=submit]').classList.add('disabled');

            if(_d.querySelector('#divisions').value !== divisionId) {
                fetch(`${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/customize/delete-all-options`,{
                    method: 'POST',
                    headers: {
                        'Accept': 'application/json',
                        'Content-Type': 'application/json'
                    }
                })
                    .then(response => response.json())
                    .then(data => {
                          console.log(data);
                    });
            }

            SaveData(formatMessage(formdata), evt.target);

            return false;
        });


        form.querySelector('#delete-form').addEventListener('click', (evt) => {
            evt.preventDefault();
            _d.querySelector('#delete-form').classList.add('disabled');
            DeleteOptions().then(() => {
                form.reset();
            });

            return false;
        });

        // form.querySelector('#clear-form').addEventListener('click', (evt) => {
        //     form.reset();
        // });

        async function DeleteOptions() {
            if (typeof wpcbSecret == 'undefined') return;

            const rawResponse = await fetch(adminEndpoint, {
                method: 'POST',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify({'allow': wpcbSecret})
            });
            const content = await rawResponse.json();
            _d.querySelector('#delete-form').classList.remove('disabled');

            return rawResponse;
        }

        async function SaveData(data, formElement) {
            const rawResponse = await fetch(endpoint, {
                method: 'POST',
                headers: {
                    'Accept': 'application/json',
                    'Content-Type': 'application/json'
                },
                body: JSON.stringify(data)
            });
            const content = await rawResponse.json();
            formElement.querySelector('input[type=submit]').classList.remove('disabled');
            console.log(content);
        }
    }

    function shortCodeGenerator(){
        const element = _d.querySelector('.jims-wpcb-shortcode-generator');

        if(element !== null){
            window.shortcodeGenerator = new SettingsShortcodeGenerator(element, jims_wpcb_variables);
            window.shortcodeGenerator.init();
        }
    }


    function populatedOptionField(target, listObject, ValueName, LabelName, placeholder) {
        let template = '<option value="{{val}}">{{label}}</option>';
        let html = [];
        html.push(`<option value="" selected>${placeholder}</option>`);

        listObject.forEach(element => {
            html.push(template.replace('{{val}}', element[ValueName]).replace('{{label}}', element[LabelName]));
        });

        target.innerHTML = html.join('');
    }

    function formatMessage(formData) {
        let message = {};
        for (var pair of formData.entries()) {
            message[pair[0]] = pair[1];
        }

        return message;
    }

    /**
     * Service Class
     *
     * @returns {{services: Promise<any>, divisions: Promise<any>}}
     */
    function service() {
        let config = {
            divisionsLink: `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/service/division`,
            servicesLink: `${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/service/division/services`
        }

        async function call(link) {
            const response = await fetch(link);
            return response.json();
        }

        return {
            'divisions': call(config.divisionsLink),
            'services': call(config.servicesLink)
        }
    }

    async function DeleteTransients() {
        /**
         * method post, path: /jims-wpcb/v1/options/clear-transients
         */
        const rawResponse = await fetch(`${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/clear-transients`, {
            method: 'POST',
            headers: {
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
        });
        const content = await rawResponse.json();
        console.log(content);

        return rawResponse;
    }

    async function UpdateLeadsRecord(){
        const rawResponse = await fetch(`${jims_wpcb_variables.home_url}/wp-json/jims-wpcb/v1/options/leads`, {
            method: 'POST',
            headers: {
                'Accept': 'application/json',
                'Content-Type': 'application/json'
            }
        });
        const content = await rawResponse.json();
        console.log(content);

        return rawResponse;
    }
}(window, document));