/**
 *
 */


/**
 * @class Autocomplete
 * @constructor
 */
class Autocomplete {

    /**
     *
     * @type {string}
     */
    className = 'jims-service-autocomplete';

    /**
     *
     * @type {string}
     */
    itemClassName = 'jims-service-autocomplete-item';

    /**
     *
     * @type {null|HtmlElement}
     */
    element = null;

    /**
     *
     * @type {null|HTMLElement}
     */
    target = null;

    /**
     *
     * @type {null|SettingServiceGrouping}
     */
    parent = null;

    autocompleteOptions = [];

    constructor(target, parent) {
        this.target = target;
        this.parent = parent;
    }

    init() {
        this.render();
        this.setBehavior();
    }

    setBehavior() {
        let input = this.target;
        if (this.target.tagName !== 'INPUT') {
            input = this.target.querySelector('input[type=text]');
        }

        input.addEventListener('keyup', (e) => {
            //open the autocomplete
            if (input.value.length < 3) return;

            this.show();

            if (Array.isArray(this.parent.serviceList.available) && this.parent.serviceList.available.length > 0) {
                this.searchOptions(input.value);
            }
        });

        this.element.addEventListener('click', (e) => {
            if (e.target.classList.contains(this.itemClassName)) {
                //find the option from data-value
                const option = this.autocompleteOptions.find((option) => {
                    return option.value === parseInt( e.target.dataset.value);
                });

                if(option) {
                    this.selected = option;
                    input.value = option.name;
                }

                this.hide();
            }
        });
    }

    setOptions(options) {
        this.options = options;
    }

    searchOptions(value) {
        if(this.autocompleteOptions.length === 0) return;

        //clear empty autocompleteOptions
        this.autocompleteOptions = this.autocompleteOptions.filter((option) => {
            return option.name !== undefined;
        });

        const filteredOptions = this.autocompleteOptions.map((option) => {

            option.element.style.display = 'none';
            if (option.name.toLowerCase().indexOf(value.toLowerCase()) !== -1) {
                option.element.style.display = 'block';
            }
        });
    }

    renderOptions() {
        if (this.parent.serviceList.available.length === 0) return;
        this.autocompleteOptions = [];

        this.parent.serviceList.available.forEach((option) => {
            const elem = document.createElement('div');
            elem.setAttribute('class', this.itemClassName);
            elem.innerHTML = option.name;
            elem.dataset.value = option.id;
            elem.style.display = 'none';

            this.autocompleteOptions.push({
                name: option.name,
                value: option.id,
                element: elem
            });

            this.element.appendChild(elem);
        });
    }

    render() {
        this.createDomElement();
        this.target.appendChild(this.element);

        this.target.style.position = 'relative';
    }

    createDomElement() {
        //jims-service-autocomplete
        const elem = document.createElement('div');
        elem.setAttribute('class', this.className);
        this.element = elem;
    }

    show() {
       this.element.innerHTML = '';
         this.renderOptions();

        this.element.style.display = 'block';
    }

    hide() {
        this.element.style.display = 'none';
    }

    onSelect(callback) {
        this.target.on('click', callback.bind(this));
    }
}

export default Autocomplete;