/**
 *
 */
import JimsServices from "../services/jims-services.mjs";
import log from "../services/log.mjs";

class SettingServiceDescriptions {

    services = [];

    constructor(element, options, parent) {
        this.element = element;
        this.options = options;
        this.parent = parent;

        this.api = new JimsServices(options.rest_url, options.bookingFormDivisionId)
    }

    init() {
        this.api.getServiceList().then((data) => {

            this.api.getServiceDescriptions().then((descriptions) => {
                descriptions = Object.values(descriptions.data);
                log.info(descriptions);

                Object.values(data).forEach((service) => {
                    let description = this.getDescription(descriptions, service);

                    let elem = document.createElement('div');
                    elem.classList.add('jims-service-panel-item','expanded');
                    elem.innerHTML = this.getItemTemplate(service.name, service.id, description);

                    this.services.push({
                        id: service.id,
                        name: service.name,
                        description: description,
                        element: elem
                    });

                    elem.querySelector('textarea').addEventListener('change', (e) => {
                        let id = e.target.dataset.id;
                        let service = this.services.find((service) => service.id == id);

                        if(service) service.description = e.target.value;
                    });

                    this.element.querySelector('.jims-service-panel-container').appendChild(elem);
                });

            });
        });
    }

    getDescription(descriptions, service) {
        let description = '';
        let serviceDescription = descriptions.find((description) => description.id == service.id);
        if (serviceDescription) description = serviceDescription.description;
        return description;
    }

    getItemTemplate(name, id, description = ''){
        return `<div class="jims-service-panel-item expanded">
                    <div class="jims-service-panel-item-title">
                    <span class="service-name">
                        ${name} <span class="service-id">
                            <small style="display:inline;">ID: ${id}</small>
                        </span>
                    </span>
                    </div>
                    <div class="jims-service-panel-item-description">
                    <textarea name="service-description" rows="1"
                              placeholder="Short Service Description" data-id="${id}" maxlength="110">${description}</textarea>
                    </div>
                </div>`;
    }

    save() {
        let data = this.services.map((service) => {
            return {
                id: service.id,
                description: service.description
            }
        });

        this.api.setServiceDescriptions(data).then((response) => {
            log.info(data, response);
        });
    }
}

export default SettingServiceDescriptions;