/**
 *
 */
import Service from "./service.mjs";

/**
 * ServiceGroup
 */
class ServiceGroup  {

    /**
     * @type {string}
     */
    services = [];

    /**
     * @type {null|HTMLElement}
     */
    element = null;


    /**
     * @type {null|SettingServiceGrouping}
     */
    parent = null;
    /**
     * @param name string
     * @param order int
     * @param parent SettingServiceGrouping
     * @param services Array|Service[]
     * @param description string
     */
    constructor(name, order, parent, services = [], description = '') {
        this.name = name;
        this.order = order;
        this.description = description;
        this.parent = parent;
        this.services = services;

        this.createDomElement();
    }

    createDomElement(){
        let template = document.createElement('div');
        template.classList.add('jims-service-panel-item');
        template.innerHTML = this.getTemplate(this.name, this.description);
        this.element = template;

        /**
         * @type {Array}
         */
        let servicesArr = this.services;

        this.services = [];
        if(servicesArr.length > 0){
            servicesArr.forEach(service => {
                this.addService(service);
            });
        }

        this.setBehavior();
    }

    setBehavior() {
        this.element.querySelector('.jims-service-panel-item-title').addEventListener('click', () => {
            this.expandToggle();
        });

        this.element.querySelector('.jims-service-panel-item-service-search a').addEventListener('click', (e) => {
            e.preventDefault();
            let input = this.element.querySelector('.jims-service-panel-item-service-search input');
            let serviceName = input.value;

            //find service by name from available services
            let service = this.parent.serviceList.available.find(s => s.name === serviceName);

            if (service) {
                console.log(service);
                this.addService({
                    name: service.name,
                    id: service.id
                });
                this.parent.setUsedService(service.id);

                input.value = '';
            }
        });

        //update description
        this.element.querySelector('.jims-service-panel-item-description textarea').addEventListener('change', (e) => {
            this.description = e.target.value;
        });

        //delete
        this.element.querySelector('.jims-service-panel-item-title .delete').addEventListener('click', (e) => {
            e.preventDefault();
            //remove all services
            this.services.forEach(service => {
                this.removeService(service);
            });

            this.parent.removeGroup(this.name);
        });
    }

    /**
     * @return {string}
     */
    expandToggle(){
        this.element.classList.toggle('expanded');
    }

    /**
     * @return {string}
     */
    getServices() {
        return this.services;
    }

    /**
     * @param service
     */
    addService(service) {
        let newService = new Service(service.name, service.id, service?.friendly_name);
        this.services.push( newService );
        this.renderService(newService );

        this.parent.setUsedService(service.id);

        newService.element.querySelector('.remove').addEventListener('click', (evt) => {
            evt.preventDefault();
            this.removeService(newService);

            console.log(this.services);
        });
    }

    removeService(service){
        service.element.remove();
        this.services = this.services.filter(s => s.id !== service.id);

        this.parent?.setAvailableService(service.id);
    }

    renderService(service){
        this.element.querySelector('.jims-service-panel-item-service-list').appendChild(service.element);
    }

    getTemplate(name, description){
        return `
        <div class="jims-service-panel-item-title">
            <i class="fa-solid fa-trash-can delete"></i>
            ${name}
            <i class="fa-solid fa-chevron-down open"></i>
        </div>
        <div class="jims-service-panel-item-content">
            <div class="jims-service-panel-item-description">
                <textarea name="category-description" rows="1" placeholder="Short List Description">${description}</textarea>
            </div>
            <div class="jims-service-panel-item-service-search">
                <input type="text" name="category-service-add" placeholder="Add Service">
                <a href="#">Add <i class="fa-solid fa-plus"></i></a>
            </div>
            <ul class="jims-service-panel-item-service-list"></ul>
        </div>`;
    }

    getObject(){
        return {
            name: this.name,
            order: this.order,
            description: this.description,
            services: this.services.map(s => s.getObject())
        }
    }
}

export default ServiceGroup;