/**
 *
 */
import JimsServices from "../../services/jims-services.mjs";
import ServiceGroup from "./service-group.mjs";
import Autocomplete from "../setting-common/autocomplete.mjs";
import serviceGroup from "./service-group.mjs";
import log from "../../services/log.mjs";

class SettingServiceGrouping {

    /**
     * @type {Object}
     */
    serviceList = {
        used: [],
        available: [],
        all: []
    };

    /**
     *
     * @type {ServiceGroup[]}
     */
    groups = [];

    /**
     * @type {JimsServices}
     */
    api = {};

    /**
     * @type {HTMLElement}
     */
    addComponent = null;

    /**
     * @type {null|Autocomplete}
     */
    autocomplete = null;

    constructor(element, options) {
        this.element = element;
        this.options = options;
        this.addComponent = this.element.querySelector('.jims-service-panel-add');

        this.api = new JimsServices(options.rest_url, options.bookingFormDivisionId);
    }

    init() {

        this.api.getServiceList().then((services) => {

            this.api.getCategories().then((categories) => {
                this.serviceList.available = this.serviceList.all = Object.values(services);

                if (categories.data.length > 0) {
                    categories.data.forEach((category) => {
                        this.addGroup(category.name, category.order, category.services, category.description);
                    });
                }
                this.setBehavior();
            });

        });
    }

    reset() {
        this.groups.forEach((group) => {
            group.element.remove();
        });
        this.groups = [];

        this.serviceList = {
            used: [],
            available: this.serviceList.all,
            all: this.serviceList.all
        };
    }

    setBehavior() {
        this.addComponent.addEventListener('click', (evt) => {
            evt.preventDefault();
            let input = this.addComponent.querySelector('input[type="text"]');

            if (input.value.length > 3) {
                this.addGroup(input.value);
                input.value = '';
            }

            return false;
        })
    }

    getAutoCompleteServiceOptions() {
        return this.serviceList.available.map((service) => {
            return {
                value: service.id,
                name: service.name
            }
        });
    }

    addGroup(name, order = 0, services = [], description = '') {
        const group = new ServiceGroup(name, order, this, services, description);
        this.groups.push(group);
        this.element.querySelector('.jims-service-panel-container').appendChild(group.element);

        let options = this.getAutoCompleteServiceOptions();
        this.autocomplete = new Autocomplete(group.element.querySelector('.jims-service-panel-item-service-search'), this);
        this.autocomplete.init();
    }

    removeGroup(name) {
        if (name.length > 0) {
            //find group by name
            let group = this.groups.find((group) => group.name.toLowerCase() == name.toLowerCase());
            if (group) {
                group.element.remove();
            }

            this.groups = this.groups.filter((group) => group.name.toLowerCase() != name.toLowerCase());
        }
    }

    /**
     *
     * @param id
     */
    setUsedService(id) {
        //set in used
        if (this.serviceList.used.indexOf(id) === -1) {
            this.serviceList.used.push(id);
        }
        //remove from available
        this.serviceList.available = this.serviceList.available.filter((value) => value.id != id);
    }

    /**
     *
     * @param id
     */
    setAvailableService(id) {
        //find service by id
        let service = this.serviceList.all.find((service) => service.id == id);

        if (service && this.serviceList.available.find(s => service.id === s.id) === undefined) {
            //set in available
            this.serviceList.available.push(service);
        }
        //remove from used
        this.serviceList.used = this.serviceList.used.filter((value) => value != id);
    }

    save() {
        let data = this.groups.map((group) => {
            return group.getObject();
        });

        this.api.setCategories(data).then((response) => {
            log.info(data, response);
        });
    }
}

export default SettingServiceGrouping;