/**
 * Created by Administrator on 2023/3/08.
 *
 */
import JimsServices from "../services/jims-services.mjs";


class SettingsShortcodeGenerator {

    shortCode = '';

    /**
     * @type {HTMLElement|null}
     */
    #HTMLElement = null;


    /**
     * @type {HTMLFormElement|null}
     */
    #formElement = null;

    /**
     * @type {{}}
     */
    #settings = {};

    /**
     * @type {[]}
     */
    divisionServices = {};

    /**
     * @type {[]}
     */
    divisions = [];

    /**
     * @type {int|null}
     */
    divisionId = null;


    serviceObjectList = [];

    allowedFormFieldNames = [
        'enable_popup',
        'show_country',
        'show_info_text',
        'contained',
        'accept_url_params',
        'allowed_url_params',
        'show_steps',
        'logo_url',
        'hide_logo',
        'division_id',
        'show_only_divisions',
        'service_ids',
        'service_list',
        'show_only_services',
        'button_text'
    ];

    /**
     *
     * @type {HTMLElement|null}
     */
    draggedService = null;


    #events = {
        'servicesLoaded': new CustomEvent('servicesLoaded', {detail: {}, bubbles: true, cancelable: true}),
        'divisionsLoaded': new CustomEvent('divisionsLoaded', {detail: {}, bubbles: true, cancelable: true}),
    };


    /**
     * @type {JimsServices|null}
     */
    #connection = null;


    #selectors = {
        'serviceListContainer': '.service-list',
        'serviceResultsContainer': '.service-list .service-results',
        'searchInput': '#service-search',
        'servicesIdsInput': '#service-ids',
        'servicesShowOnly': '#show-only-services',
        'divisionSelect': 'select[name="division_id"]',
        'showDivisionsOnly': '#show-divisions-only',
        'hideHeaderToggle': '.hide-header',
        'shortcodeResult': '#native-booking-form-shortcode'
    };

    #parts = {
        'serviceListContainer': document.querySelector(this.#selectors.serviceListContainer),
        'serviceResultsContainer': document.querySelector(this.#selectors.serviceResultsContainer),
        'searchInput': document.querySelector(this.#selectors.searchInput),
        'servicesIdsInput': document.querySelector(this.#selectors.servicesIdsInput),
        'servicesShowOnly': document.querySelector(this.#selectors.servicesShowOnly),
        'divisionSelect': document.querySelector(this.#selectors.divisionSelect),
        'showDivisionsOnly': document.querySelector(this.#selectors.showDivisionsOnly),
        'hideHeaderToggle': document.querySelector(this.#selectors.hideHeaderToggle),
        'shortcodeResult': document.querySelector(this.#selectors.shortcodeResult)
    };

    constructor(element, settings) {
        this.#HTMLElement = element;
        this.#settings = {...settings};
        this.divisionId = this.#settings.bookingFormDivisionId;

        this.#connection = new JimsServices(
            this.#settings.rest_url,
            this.#settings.bookingFormDivisionId,
            this.#settings.country
        );
    }

    init() {
        this.#formElement = this.getFormElement();
        this.shortCode = this.#formElement.querySelector('#shortcode-name').value;

        this.loadDivisions();
        this.loadServices();

        this.events();
    }

    events() {
        if (this.#formElement === null) {
            return;
        }

        this.#parts.servicesIdsInput.addEventListener('drop', this.serviceDragDrop);
        this.#parts.servicesIdsInput.addEventListener('dragover', this.serviceDragDrop);

        this.#parts.servicesShowOnly.addEventListener('drop', this.serviceDragDrop);
        this.#parts.servicesShowOnly.addEventListener('dragover', this.serviceDragDrop);


        this.#parts.searchInput.addEventListener('input', (evt) => {
            let search = evt.target.value;

            if (search.length > 0) {
                let filteredServices = [];
                this.serviceObjectList.forEach((service) => {
                    if (service.friendly_name.toLowerCase().includes(search.toLowerCase())) {
                        filteredServices.push(service.id);
                    }
                });

                if (filteredServices.length > 0) {
                    this.serviceObjectList.forEach((service) => {
                        if (filteredServices.includes(service.id)) {
                            service.htmlElement.classList.remove('hidden');
                        } else {
                            service.htmlElement.classList.add('hidden');
                        }
                    });
                }
            } else {
                //remove hidden class from all
                this.serviceObjectList.forEach((service) => {
                    service.htmlElement.classList.remove('hidden');
                });
            }
        });

        this.#parts.hideHeaderToggle.addEventListener('click', (evt) => {
           if(evt.target.checked) {
              this.#formElement.querySelector('.form-group.logo').classList.add('disabled');
           } else {
               this.#formElement.querySelector('.form-group.logo').classList.remove('disabled');
           }
        });

        this.#formElement.querySelector('select.steps').addEventListener('change', (evt) => {
            let value = evt.target.value;
            console.log(value);

            if(value.includes('divisions')) {
                this.#parts.showDivisionsOnly.disabled = false;
            } else {
                this.#parts.showDivisionsOnly.disabled = true;
            }
        });

        /**
         * Listen for form submit
         */
        this.#formElement.addEventListener('submit', (evt) => {
            evt.preventDefault();

            let properties = [];
            let processedProperties = {};

            let form = evt.target;
            let formData = new FormData(form);
            //show all entries
            let entries = formData.entries();
            //loop through all entries
            for (let entry of entries) {
                let [name, value] = entry;

                //value is empty skip
                if (value === '') {
                    continue;
                }

                if (this.allowedFormFieldNames.includes(name)) {

                    //if name show_country and value is on or off, change off to on and on to off
                    if(name === 'show_country' || name === 'show_info_text') {
                        let AvailableValues = ['on', 'off'];
                        AvailableValues.splice(['on', 'off'].indexOf(value), 1);
                        value = AvailableValues[0];
                    }

                    //if name is division_id and the division_id is the same as in main settings, don't add it to the shortcode
                    if(name === 'division_id' && parseInt(value) === parseInt(this.#settings.bookingFormDivisionId)) {
                        continue;
                    }

                    value = ['on', 'off'].indexOf(value) > -1 ? this.transformToggleValue(value) : value;

                    if(Object.keys(processedProperties).includes(name)) {
                        processedProperties[name] = `${processedProperties[name]},${value}`;
                        value = processedProperties[name];

                        //remove duplicate values
                        properties = properties.filter((property) => {
                            return property.indexOf(`${name}`) === -1;
                        });
                    } else {
                        processedProperties[name] = value;
                    }

                    if (name === 'show_steps'){
                        value = [value, 'services', 'client_info'].join(',');
                    }

                    if(processedProperties['contained'] !== undefined && processedProperties['contained'] === 1) {
                        if(name.includes('logo')) {
                            continue;
                        }
                    }

                    if(processedProperties['show_steps'] !== undefined && !processedProperties['show_steps'].includes('divisions')) {
                        if(name === 'show_only_divisions') {
                            continue;
                        }
                    }

                    properties.push(`${name}="${value}"`);
                }
            }

            this.#parts.shortcodeResult.innerText = `[${this.shortCode} ${properties.join(' ')}]`;

            return false;
        });

        this.#HTMLElement.addEventListener('servicesLoaded', (evt) => {
            this.renderServiceList();
        });

        this.#HTMLElement.addEventListener('divisionsLoaded', (evt) => {
            this.renderDivisionList();
        });
    }

    /**
     * @returns {HTMLElement|null}
     */
    getFormElement() {
        let form = this.#HTMLElement.querySelector('form');

        return form !== null ? form : null;
    }

    loadServices() {
        this.#connection.getServiceList(this.divisionId).then((services) => {

            if (Object.values(services).length > 0) {
                this.divisionServices[this.divisionId] = Object.values(services);

                this.#HTMLElement.dispatchEvent(this.#events.servicesLoaded);
            }
        });
    }

    renderServiceList() {
        if (this.#parts.serviceResultsContainer === null) {
            return;
        }

        if (this.#parts.serviceResultsContainer.querySelector('ul') !== null) {
            this.#parts.serviceResultsContainer.querySelector('ul').remove();
        }

        let services = this.divisionServices[this.divisionId];

        if (services.length > 0) {
            //remove empty class
            this.#parts.serviceResultsContainer.classList.remove('empty');

            //render list
            let list = document.createElement('ul');

            services.forEach((service) => {
                let listItem = document.createElement('li');
                listItem.textContent = service.friendly_name;
                listItem.dataset.serviceId = service.id;
                listItem.draggable = true;

                this.serviceObjectList.push({
                    ...service,
                    htmlElement: listItem
                });

                list.appendChild(listItem);

                listItem.addEventListener('dragstart', this.serviceDragStart);
                listItem.addEventListener('dragend', this.serviceDragEnd);
            });

            this.#parts.serviceResultsContainer.appendChild(list);
        }
    }

    renderDivisionList() {
        this.#parts.divisionSelect.innerHTML = '';
        this.#parts.showDivisionsOnly.innerHTML = '';

        if (this.divisions.length > 0) {
            let select = this.#parts.divisionSelect;
            let onlyDivisions = this.#parts.showDivisionsOnly;

            if (select !== null) {
                this.divisions.forEach((division) => {
                    let option = this.createDivisionListOption(select, division);
                    let onlyDivisionOption = this.createDivisionListOption(onlyDivisions, division);

                    if ( parseInt( division.id ) === parseInt( this.divisionId ) ) {
                        option.selected = true;
                    }
                });

                select.addEventListener('change', (evt) => {
                    this.divisionId = evt.target.value;
                    this.loadServices();
                });

                this.#parts.divisionSelect.disabled = false;
            }
        }
    }

    createDivisionListOption(selectElement, division) {
        let option = document.createElement('option');
        option.value = division.id;
        option.textContent = division.name;

        selectElement.appendChild(option);
        return option;
    }

    serviceDragStart(evt) {
        evt.target.style.opacity = '0.4';

        evt.dataTransfer.clearData();
        // Set the drag's format and data.
        // Use the event target's id for the data
        evt.dataTransfer.setData("text/plain", evt.target.dataset.serviceId);
    }

    serviceDragEnd(evt) {
        evt.target.style.opacity = '1';
    }

    serviceDragDrop(evt) {
        evt.preventDefault();
        const serviceId = evt.dataTransfer.getData("text");

        if (evt.target.nodeName === 'INPUT') {
            if (evt.target.value === '') {
                evt.target.value = serviceId;
            } else {
                let values = evt.target.value.split(',');

                //remove empty values
                values = values.filter((value) => {
                    return value !== '';
                });

                if (!values.includes(serviceId)) {
                    values.push(serviceId);
                    evt.target.value = values.join(',');
                }
            }
        }
    }

    loadDivisions() {
        this.#parts.divisionSelect.innerHTML = '<option value="">Loading...</option>';
        this.#parts.divisionSelect.disabled = true;

        this.#connection.getDivisionList().then((divisions) => {
            if (divisions.length > 0) {
                this.divisions = divisions;

                this.#HTMLElement.dispatchEvent(this.#events.divisionsLoaded);
            }
        });
    }

    transformToggleValue(value) {
        return value === 'on' ? 1 : 0;
    }
}

export default SettingsShortcodeGenerator;