import Toggle from "./toggle.mjs";
import Notification from "./notification.mjs";

class HtmlExtensionRegistry
{
    prefix = "wpcb-";

    #availableExtensions = {
        "toggle": {
            prototype: Toggle
        },
        "notification": {
            prototype: Notification,
        }
    }

    #list = [];

    constructor(){}

    register(tagName){
        this.#list.push(tagName);
    }

    init(){
        this.#list.forEach((name) => {

            // Check if the element is already defined
            if(customElements.get(this.getTag(name)) !== undefined){
                return;
            }

            // Check if the element extends another element
            let configuration = this.#availableExtensions[name]?.extends ? {
                extends: this.#availableExtensions[name].extends
            }: {};

            // Register the element
            customElements.define(this.getTag(name), this.#availableExtensions[name].prototype, configuration);
        });
    }

    get list(){
        return this.#list;
    }


    getTag(tagName){
        return this.prefix + tagName;
    }
}

export default HtmlExtensionRegistry;