class Notification extends HTMLElement {

    observedAttributes = ["type"];

    #style = null;

    #parts = {
        text: document.createElement("div"),
    }

    constructor() {
        super();
        const shadow = this.attachShadow({mode: "open"});

        // Optionally, you can still use slots for other content
        // shadow.innerHTML = `<slot name="fallback"></slot>`;
        this.#parts.text.textContent = '';
        this.shadowRoot.appendChild(this.#parts.text);
    }

    connectedCallback() {
        this.#setStyle();

        const contentDiv = this.#parts.text;

        if(this.attributes?.type !== null && this.attributes?.type !== undefined ) {
            console.log(this.attributes?.type);
            this.#parts.text.classList.add(this.attributes.type.value);
        } else {
            this.#parts.text.classList.add("info");
        }

        setTimeout(() => {
            contentDiv.innerHTML = '';
            this.childNodes.forEach((node) => {
                contentDiv.appendChild(node.cloneNode(true));
            });
        }, 10);
    }

    disconnectedCallback() {
        // console.log("Custom element removed from page.");
    }

    adoptedCallback() {
        // console.log("Custom element moved to new page.");
    }

    attributeChangedCallback(name, oldValue, newValue) {
        // console.log(`Attribute ${name} has changed.`);

        if(name === "type") {
            this.#parts.text.classList.remove(oldValue);
            this.#parts.text.classList.add(newValue);
        }
    }

    #removeTabsAndNewLines(text) {
        return text.replace(/\t/g, "").replace(/\n/g, "");
    }

    #setStyle() {
        this.#style = document.createElement("style");
        this.#style.textContent = `
      div {
      box-sizing: border-box;
          padding: 10px;
          color: white;
        display: block;
        flex-direction: column;
        align-items: center;
        justify-content: center;
        width: 100%;
        height: 100%;
        background-color: inherit;
        line-height: 1.5;
      }
      
      div.error {
        background-color: #f44336;
      }
      
        div.warning {
            background-color: #ffeb3b;
            color: darkslategray;
        }
        
        div.success {
            background-color: #4caf50;
        }
        
        div.info {
            background-color: #2196F3FF;
        }
    `;

        this.shadowRoot.appendChild(this.#style);
    }

    set type(value) {
        this.setAttribute("type", value);
    }

    get type() {
        return this.getAttribute("type");
    }
}

export default Notification;