class Toggle extends HTMLElement {

    static formAssociated = true;

    observedAttributes = ["text", "checked"];

    #parts = {
        containerLabel: document.createElement("label"),
        title: document.createElement("span"),
        switchElement: document.createElement("span"),
        input: document.createElement("input"),
        spanSlider: document.createElement("span")
    }

    #style = null;

    #target = null;

    constructor() {
        super();
        this.internals_ = this.attachInternals();

        const shadow = this.attachShadow({mode: "open"});

        const containerLabel = this.#createHTML();
        this.shadowRoot.appendChild(this.#parts.title);
        shadow.appendChild(containerLabel);
        this.#setStyle();
    }

    connectedCallback() {
        const text = this.attributes.text.value;
        this.#parts.title.innerText = text;

        if (this.attributes?.checked?.value === "true" || (this.attributes?.checked && this.attributes?.checked?.value !== "false")) {
            this.#parts.input.checked = true;
        }

        if (this.attributes?.for !== undefined) {
            document.addEventListener("DOMContentLoaded", () => {
                let target = document.querySelector(`#${this.attributes.for.value}`);
                if (target !== null && target !== undefined) {
                   target.disabled = !this.#parts.input.checked;
                   this.target = target;
                }
            });
        }


        this.#parts.input.addEventListener("change", () => {

            if(this.hasAttribute("checked")){
                this.attributes.checked.value = this.#parts.input.checked;
            } else {
                this.setAttribute("checked", this.#parts.input.checked);
            }

            this.internals_.setFormValue(this.#parts.input.checked ? "on" : "off");

            this.dispatchEvent(new Event("change", {bubbles: true}));
        });


        this.addEventListener("change", () => {
            if(this.target !== null && this.target !== undefined){
                this.target.disabled = !this.#parts.input.checked;
            }
        });
    }

    disconnectedCallback() {
        // console.log("Custom element removed from page.");
    }

    adoptedCallback() {
        // console.log("Custom element moved to new page.");
    }

    attributeChangedCallback(name, oldValue, newValue) {
        //console.log(`Attribute ${name} has changed.`);

        if(name === 'checked'){
            this.target.disabled = !this.#parts.input.checked;
        }

    }

    #createHTML() {
        this.#parts.switchElement.classList.add("switch");
        this.#parts.input.type = "checkbox";
        this.#parts.spanSlider.classList.add("slider");
        this.#parts.title.style.verticalAlign = "-webkit-baseline-middle";

        this.#parts.switchElement.appendChild(this.#parts.input);
        this.#parts.switchElement.appendChild(this.#parts.spanSlider);

        this.#parts.containerLabel.appendChild(this.#parts.switchElement);

        return this.#parts.containerLabel;
    }

    #setStyle() {
        this.#style = document.createElement("style");
        this.#style.textContent = `
        label {
                position: relative;
              display: inline-block;
              width: 60px;
              height: 34px;
              vertical-align: inherit;
              margin-left: 10px;
        }
        
        input {
            opacity: 0;
            width: 0;
            height: 0;
          }

          .slider {
            position: absolute;
            cursor: pointer;
            top: 0;
            left: 0;
            right: 0;
            bottom: 0;
            background-color: #ccc;
            -webkit-transition: .4s;
            transition: .4s;
            border-radius: 34px;
          }

          .slider:before {
            position: absolute;
            content: "";
            height: 26px;
            width: 26px;
            left: 4px;
            bottom: 4px;
            background-color: white;
            -webkit-transition: .4s;
            transition: .4s;
            border-radius: 50%;
          }
        
          input:checked + .slider {
            background-color: #2196F3FF;
          }
        
          input:focus + .slider {
            box-shadow: 0 0 1px #2196F3FF;
          }
        
      
          input:checked + .slider:before {
            -webkit-transform: translateX(26px);
            -ms-transform: translateX(26px);
            transform: translateX(26px);
          }
        `;

        this.shadowRoot.appendChild(this.#style);
    }

    get value() {
        return this.#parts.input.checked;
    }

    set value(value) {
        this.#parts.input.checked = value;
    }

    get checked() {
        return this.#parts.input.checked;
    }

    set checked(value) {
        this.#parts.input.checked = value;
    }

    set name(value) {
        this.attributes.name = value;
    }

    get name() {
        return this.attributes?.name;
    }

    set for(value) {
        this.attributes.for = value;
    }

    get for() {
        return this.attributes?.for;
    }


}

export default Toggle;