<?php

namespace JimsCustomerBooking\Admin\BackendPages;

use JimsCustomerBooking\Template;

if( ! class_exists('JimsCustomerBooking\Admin\BackendPages\BackendPage')){
    /**
     *
     */
    abstract class BackendPage {

        /**
         * @var string
         */
        protected $name = 'backend_page';

        /**
         * @var string
         */
        protected $title = '';

        /**
         * This appears only in the pages header
         * @var string $sub_title
         */
        protected $sub_title = '';

        /**
         * @var string
         */
        protected $menu_title = '';

        /**
         * @var string
         */
        protected $capability = 'manage_options';


        /**
         * @var string
         */
        protected $parent_slug = false;

        /**
         * @var string
         */
        protected $menu_slug = '';

        /**
         * @var string
         */
        protected $icon_path = '';

        /**
         * @var string
         */
        protected $menu_icon = 'dashicons-screenoptions';

        /**
         * @var int
         */
        protected $position = 30;

        /**
         * @var string
         */
        protected $template_name = 'default_backend_page';


        protected $hook = '';


        protected $hide_menu_item = false;

        /**
         * BackendPage constructor.
         */
        public function __construct()
        {
            add_action('admin_menu', [$this, 'Setup']);

            if($this->hide_menu_item){
                add_action('admin_head', [$this, 'HideMenuItem']);
            }
        }

        public function Setup()
        {
            if (!empty($this->parent_slug)) {
                return $this->SetupSubMenuPage();
            }
            return $this->SetupMenuPage();
        }

        /**
         * @return bool
         */
        private function SetupMenuPage()
        {
            $this->hook = \add_menu_page(
                $this->GetTitle(),
               $this->getMenuTitle (),
                $this->capability,
                $this->GetMenuSlug(),
                [$this, 'GetContent'],
                $this->GetMenuIcon(),
                $this->position
            );

            return \strlen ($this->hook) > 0;
        }

        /**
         * @return bool
         */
        private function SetupSubMenuPage()
        {
            if (is_string($this->parent_slug))
                $this->hook = \add_submenu_page(
                    $this->parent_slug,
                    $this->GetTitle(),
                    $this->getMenuTitle(),
                    $this->capability,
                    $this->GetMenuSlug(),
                    [$this, 'GetContent'],
                    $this->position);

            return \strlen ($this->hook) > 0;
        }

        abstract public function GetContent();

        /**
         * @return string
         */
        public function GetName(): string
        {
            return $this->name;
        }

        /**
         * @return string
         */
        public function GetTitle(): string
        {
            if (empty($this->title)) {
                return preg_replace('#[-_]#', ' ', $this->name);
            }
            return $this->title;
        }

        /**
         * @return string
         */
        public function GetMenuSlug($only_property = false): string
        {
            if($only_property) return $this->menu_slug;

            if (empty($this->menu_slug)) {
                return "{$this->name}/{$this->name}.php";
            }
            return $this->menu_slug;
        }

        public function getHook(): string
        {
            return $this->hook;
        }

        public function HideMenuItem()
        {
            global $submenu;

            if (isset($submenu[$this->parent_slug])) {
                foreach ($submenu[$this->parent_slug] as $key => $menu_item) {
                    if ($menu_item[2] == $this->GetMenuSlug(true)) {
                        unset($submenu[$this->parent_slug][$key]);
                    }
                }
            }
        }

        /**
         * @return Template|string
         */
        protected function GetTemplate()
        {
            return new Template($this->template_name);
        }

        /**
         * @return string
         */
        protected function GetMenuIcon()
        {
            return empty($this->icon_path) ? plugins_url('/assets/dist/img/jims20x20.png', JIMS_CBP_FILE) : $this->menu_icon;
        }

        /**
         * @return string
         */
        protected function getMenuTitle(){
            return empty($this->menu_title) ? ucwords($this->GetTitle()) : ucwords($this->GetTitle());
        }

    }
}