<?php

namespace JimsCustomerBooking\Database;

use JimsCustomerBooking\Jims_Plugin;

/**
 * Class Model
 *
 * Database Model
 * use the wpdb class to interact with the database
 *
 *
 * @package JimsCustomerBooking\Database
 */
abstract class Model
{
    /**
     * @var int
     */
    const RESULT_TYPE_DEFAULT = ARRAY_N;

    const RESULT_TYPE_OBJECT = OBJECT;

    const RESULT_TYPE_ARRAY_A = ARRAY_A;

    const RESULT_TYPE_ARRAY_N = ARRAY_N;

    const RESULT_TYPE_OBJECT_K = OBJECT_K;

    /**
     * @var \wpdb $db
     */
    private $db;

    protected $table_name = '';

    protected $results_limit = 25;

    /**
     * Model constructor.
     */
    public function __construct()
    {
        global $wpdb;
        $this->db = $wpdb;
    }

    /**
     * @return string
     */
    public function getTable()
    {
        return $this->db->prefix . implode ('_', [Jims_Plugin::PREFIX, $this->table_name] );
    }

    public function prepare($sql, $args = [])
    {
        return $this->db->prepare($sql, $args);
    }

    /**
     * @param $sql
     * @param $args
     * @return bool|int|\mysqli_result|resource|null
     */
    public function query($sql, $args = [])
    {
        $sql = $this->db->prepare($sql, $args);
        return $this->db->query($sql);
    }

    public function update($table, $data, $where, $format = null, $where_format = null)
    {
        $this->db->update($table, $data, $where, $format, $where_format);
    }

    /**
     * @param $table_name
     * @param $data
     * @return int
     */
    public function insert($data)
    {
        $table = $this->getTable();
        $this->db->insert($table, $data);

        return $this->db->insert_id;
    }

    public function select($fields = '*', $where = [], $operator = 'AND', $limit = 25, $offset = 0, $order_by = 'id', $order = 'DESC')
    {
        $table = $this->getTable();
        $fields = $fields != '*' ? implode (',', $fields) : $fields;

        list($where, $directive_string) = $this->getSelectConditions ($where, $operator, $limit, $offset, $order_by, $order);

        return $this->get_results ("SELECT {$fields} FROM $table".$where.$directive_string);
    }

    /**
     * @param $sql
     * @return array|object|\stdClass[]|null
     */
    public function get_results($sql, $output = self::RESULT_TYPE_ARRAY_A)
    {
        return $this->db->get_results($sql, $output);
    }

    /**
     * @param $table_name
     * @param $id
     * @return array|object|\stdClass|null
     */
    public function getFromId($id, $output = self::RESULT_TYPE_OBJECT)
    {
        $table = $this->getTable();
        $sql = "SELECT * FROM $table WHERE id = $id";
        return $this->db->get_row($sql, $output);
    }

    /**
     * @param $timestamp
     * @return string
     */
    protected function getAsDatetime($timestamp)
    {
        return date('Y-m-d H:i:s', $timestamp);
    }

    public function getDateField($field_name)
    {
        $this->getAsDatetime ($field_name);
    }

    /**
     * @param mixed $where
     * @param mixed $operator
     * @param mixed $limit
     * @param mixed $offset
     * @param mixed $order_by
     * @param mixed $order
     * @return array
     */
    protected function getSelectConditions(mixed $where, mixed $operator, mixed $limit, mixed $offset, mixed $order_by, mixed $order): array
    {
        if (!empty($where) && is_array ($where)) {
            $where = sprintf (' WHERE %s', implode (" {$operator} ", $where));
        } else {
            $where = '';
        }

        if($limit == 0 || empty($limit)) {
            $limit = $this->results_limit;
        }

        $directive_string = sprintf (' LIMIT %d', $limit);

        if ($offset > 0) {
            $directive_string .= sprintf (' OFFSET %d', $offset);
        }

        $directive_string = sprintf (' ORDER BY %s %s', $order_by, $order);
        return array($where, $directive_string);
    }

    public function __call($name, $arguments)
    {
        if (method_exists ($this->db, $name)) {
            return call_user_func_array ([$this->db, $name], $arguments);
        }
    }
}