<?php

namespace JimsCustomerBooking\Helpers;

if (!class_exists ('JimsCustomerBooking\Helpers\Http')) {

    /**
     *
     */
    class Http extends Helper
    {

        /**
         * @param string $url
         * @param array $params
         * @return array
         */
        public static function Get(string $url, array $params = [])
        {
            return self::Request ($url, 'GET', $params);
        }

        /**
         * @param string $url
         * @param array $params
         * @param array $extra_headers
         * @return array
         */
        public static function Post(string $url, array $params = [], array $extra_headers = [])
        {
            return self::Request ($url, 'POST', $params, $extra_headers);
        }


        /**
         * @param string $url
         * @param string $verb
         * @param array $params
         * @param array $extra_headers
         * @return array
         */
        public static function Request(string $url, string $verb = 'GET', array $params = [], array $extra_headers = [], $timeout = false)
        {
            //use curl to make the request
            $connection = curl_init ();

            $headers = self::GetHeadersForContentType ($extra_headers['Content-Type'] ?? 'application/json');


            if(empty($timeout)){
                $timeout = 0;
            }

            //set the url, number of POST vars, POST data with opt array
            curl_setopt_array ($connection, [
                CURLOPT_URL => $url,
                CURLOPT_RETURNTRANSFER => true,
                CURLOPT_ENCODING => '',
                CURLOPT_TIMEOUT => $timeout,
                CURLOPT_HTTPHEADER => array_merge ($headers, $extra_headers),
                CURLOPT_POSTFIELDS => self::getBodyForContentType ($extra_headers['Content-Type'] ?? 'application/json', $params),
                CURLOPT_CUSTOMREQUEST => $verb,
            ]);

            //execute post
            $result = curl_exec ($connection);

            return [
                'result' => $result,
                'headers' => curl_getinfo ($connection),
                'error' => curl_error ($connection),
            ];
        }

        /**
         * This method tests if the API is available
         *
         * @return boolean
         */
        public static function makeTestRequest($url){
            $result = self::Request($url, 'GET', [], [], 10);
            return $result['headers']['http_code'] == 200;
        }


        /**
         * @param $content_type
         * @return string[]
         */
        public static function GetHeadersForContentType($content_type)
        {
            switch ($content_type) {
                case 'application/json':
                    return [
                        'Content-Type: application/json',
                        'Accept: application/json',
                    ];
                case 'application/x-www-form-urlencoded':
                    return [
                        'Content-Type: application/x-www-form-urlencoded',
                        'Accept: application/json',
                    ];
                default:
                    return [
                        'Content-Type: application/json',
                        'Accept: application/json',
                    ];
            }
        }

        /**
         * @param $content_type
         * @param $params
         * @return string
         */
        public static function getBodyForContentType($content_type, $params)
        {
            switch ($content_type) {
                case 'application/json':
                    return \json_encode ($params);
                case 'application/x-www-form-urlencoded':
                    return \http_build_query ($params);
                default:
                    return \json_encode ($params);
            }
        }

        public static function getUserIP()
        {
            if (!empty($_SERVER['HTTP_CLIENT_IP'])) {
                $ip = $_SERVER['HTTP_CLIENT_IP'];
            } elseif (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])) {
                $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
            } else {
                $ip = $_SERVER['REMOTE_ADDR'];
            }

            return $ip;
        }
    }
}