<?php

namespace JimsCustomerBooking\Helpers;

use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Jims_Options;
use JimsCustomerBooking\Jims_Plugin;
use JimsCustomerBooking\Services\TokenManager;

if( ! class_exists('JimsCustomerBooking\Helpers\OptionHelper')){
    /**
     *
     */
    class OptionHelper extends Helper {

        /**
         * @param $option_name
         * @param $default
         * @return false|mixed|void
         */
        public static function get($option_name, $default = null){
            return get_option( StringHelper::GetPrefixed($option_name, 'machine'), $default );
        }

        public static function getValue($option_name, $value_name, $default = null){
            $option = self::get($option_name, []);
            return $option[$value_name] ?? $default;
        }

        /**
         * @return Jims_Options
         */
        public static function getEmbeddedFormOptions() {
            $default = new Jims_Options();
            $options = self::get('form', $default);

            if ($options instanceof Jims_Options) {
                return $options;
            }

            $unserializedOptions = unserialize($options);
            return $unserializedOptions instanceof Jims_Options ? $unserializedOptions : $default;
        }

        /**
         * @param $script_name
         * @return void
         */
        public static function localizeFormVariables($script_name){
            $settings = static::get(NativeFormSettingsPage::$option_name, []);

            //remove options with key starting with 'required_fields'
            $settings = array_filter($settings, function($key){
                return !str_contains($key, 'required_fields');
            }, ARRAY_FILTER_USE_KEY);

            $settings['requiredFields'] = self::getRequiredInfoFields();

            wp_localize_script(
                StringHelper::GetPrefixed($script_name),
                StringHelper::GetPrefixed('variables', 'machine'),
                array_merge(
                    [
                        'home_url' => get_bloginfo('wpurl'),
                        'version' => Jims_Plugin::$version,
                        'wpcb_url' => JIMS_CBP_URL,
                        'rest_url' => get_rest_url (),
                        'secret' => wp_create_nonce('wp_rest'),
                        'session_token' => TokenManager::encode(Jims_Plugin::SECKEY, Jims_Plugin::PREFIX . TokenManager::generateRandomString(10), '10 minutes'),
                    ],
                    $settings,
                    ['embeddedFormOptions' => Jims_Plugin::getOptions()->getProperties()],
                )
            );
        }

        public static function getArrayFirstValue(array $arr){
            return array_values($arr)[0] ?? null;
        }

        public static function getFormOption($name, $default = ''){
            $options = self::get(NativeFormSettingsPage::$option_name, [] );
            return $options[$name] ?? $default;
        }

        public static function getRequiredInfoFields(){
            $options = OptionHelper::get(NativeFormSettingsPage::$option_name, [] );

            $fields =  array_filter( array_keys($options), function($field){
                return str_contains($field, 'required_fields');
            });

            $fields[] = 'address';

            if(count($fields) <= 1 ) {
                return  [
                    'address',
                    'firstname',
                    'lastname',
                    'email',
                    'phone',
                ];
            }

            return array_map(function($field){
                return str_replace('required_fields-', '', $field);
            }, $fields);
        }
    }
}