<?php

namespace JimsCustomerBooking\Helpers;

use JimsCustomerBooking\Jims_Plugin;
use JimsCustomerBooking\Validators\JsonValidator;
use JimsCustomerBooking\Validators\Validator;

if (!class_exists ('JimsCustomerBooking\Helpers\StringHelper')) {
    /**
     *
     */
    class StringHelper extends Helper
    {


        const PREFIXED_TYPE_SLUG = 'slug';

        const PREFIXED_TYPE_MACHINE = 'machine';

        /**
         * @param string $string
         * @param string $type , slug|machine
         * @return string
         */
        public static function GetPrefixed($string, $type = self::PREFIXED_TYPE_SLUG)
        {
            list($prefix, $replacement) = self::_getReplacementValues ($type);

            $result = preg_replace ('#[\-_ ]#', $replacement, "{$prefix} {$string}");
            return is_string ($result) ? strtolower ($result) : strtolower (implode ($replacement, $result));
        }

        public static function getPrefix($type = self::PREFIXED_TYPE_SLUG)
        {
            list($prefix, $replacement) = self::_getReplacementValues ($type);
            return preg_replace ('#[\-_ ]#', $replacement, "{$prefix}");
        }

        /**
         * Same as GetPrefixed but takes an array of parts
         *
         * @param array $parts
         * @param $type
         * @return string
         */
        public static function GetPrefixedFromParts($parts, $type = self::PREFIXED_TYPE_SLUG)
        {
            $string = implode (' ', $parts);
            return self::GetPrefixed ($string, $type);
        }

        /**
         * @param $type
         * @return array
         */
        public static function _getReplacementValues($type)
        {
            return [
                Jims_Plugin::PREFIX,
                [self::PREFIXED_TYPE_SLUG => '-', self::PREFIXED_TYPE_MACHINE => '_'][$type]
            ];
        }

        /**
         * @param $string
         * @return bool
         */
        public static function isJson($string) {
          return JsonValidator::isJson($string);
        }

        /**
         * @param $string
         * @return bool
         */
        public static function isSerialized($string) {
            return (@unserialize($string) !== false);
        }

        /**
         * @param $version1 string, format 1.0.0, follows the regex pattern \d+(\.\d+){0,2}
         * @param $version2 string, format 1.0.0, follows the regex pattern \d+(\.\d+){0,2}
         * @return bool, true if version1 is greater than version2
         */
        public static function compareVersionStrings($version1, $version2)
        {
            //before version compare we need to format the version strings, in case they are not formatted
            $version1 = explode ('.', $version1);
            $version2 = explode ('.', $version2);

            //convert every version part to an integer
            $version1 = array_map ('intval', $version1);
            $version2 = array_map ('intval', $version2);

            //pad the version arrays with 0s
            $version1 = array_pad ($version1, 3, 0);
            $version2 = array_pad ($version2, 3, 0);

            //recompose the version strings
            $version1 = implode ('.', $version1);
            $version2 = implode ('.', $version2);

            //compare the versions
            return version_compare ($version1, $version2, '>');
        }

        public static function getNewZealandSpecialCharacters(){
            return [
                "ā" => "a",
                "ē" => "e",
                "ī" => "i",
                "ō" => "o",
                "ū" => "u",
                "Ā" => "A",
                "Ē" => "E",
                "Ī" => "I",
                "Ō" => "O",
                "Ū" => "U",
                "-" => "-"
            ];
        }

        /**
         * @param $string
         * @return string
         */
        public static function convertSpecialNZSpecialChars($string){
            return strtr($string, self::getNewZealandSpecialCharacters());
        }

        /**
         * @param $array
         * @return array
         */
        public static function convertSpecialNZCharactersInArrayValues($array){
           $result_array = [];
           foreach ($array as $key => $value){
               $result_array[$key] = self::convertSpecialNZSpecialChars($value);
           }

           return $result_array;
        }

        public static function slugify($string){
            $string = strtolower($string);
            $string = preg_replace('/[^a-z0-9 -]+/', '', $string);
            $string = preg_replace('/[\s-]+/', '-', $string);
            return trim($string, '-');
        }
    }

}