<?php

namespace JimsCustomerBooking\Integrations;

use JimsCustomerBooking\Database\Booking as Booking;
use JimsCustomerBooking\Helpers\OptionHelper;
use WP_Error;

/**
 * Class JimsLeadsIntegration
 *
 * @package JimsCustomerBooking\Integrations
 */
class JimsLeadsIntegration
{
    private $host = 'https://api.leads.jims.net';

    private $endpoints = [
        'update_site' => '/wp-json/jims-leads/v1/statistics/website',
        'update_realtime' => '/wp-json/jims-leads/v1/statistics/realtime',
        'email_failed' => '/wp-json/jims-leads/v1/email/failed'
    ];

    /**
     * @param $division_id
     * @param $country
     * @return array|WP_Error
     */
    public function updateSite($division_id, $country, $debugMode = 0)
    {
        $url = $this->getEndpointUrl('update_site');
        $site_name = get_bloginfo('name');

        if(empty($site_name)){
            //use site url to extract site name
            $site_name = parse_url(get_bloginfo('url'), PHP_URL_HOST);
        }

        $data = [
            'token' => 'bypass',
            'id' => (int) $division_id,
            'country' => strtolower($country),
            'name' => $site_name,
            'link' => get_bloginfo('url'),
            'version' => JIMS_CBP_VERSION,
            'test_mode' => $debugMode
        ];

        error_log(json_encode($data), 0);
        return $this->sendRequest($url, $data);
    }

    /**
     * @param $booking_id
     * @return array|WP_Error
     */
    public function updateRealTimeBooking( $booking_id, $error_code = null ){
        $debug_mode = OptionHelper::getFormOption('debugMode', 0);

        if($debug_mode == 1){
            return [];
        }

        $url = $this->getEndpointUrl('update_realtime');
        $booking = new Booking();
        $booking_data = $booking->getFromId( $booking_id, Booking::RESULT_TYPE_ARRAY_A );

        if(!empty($error_code)){
            $booking_data['error_code'] = $error_code;
        }

        //add token bypass to $booking_data
        $booking_data['token'] = 'bypass';

        return $this->sendRequest( $url, $booking_data );
    }

    /**
     * @param $booking_id
     * @return array|WP_Error
     */
    public function sendErrorNotificationEmail($booking_id, $error_code = null){
        $debug_mode = OptionHelper::getFormOption('debugMode', 0);

        if($debug_mode == 1){
            return [];
        }

        $url = $this->getEndpointUrl('email_failed');
        $booking = new Booking();
        $booking_data = $booking->getFromId( $booking_id, Booking::RESULT_TYPE_ARRAY_A );

        //add token bypass to $booking_data
        $booking_data['token'] = 'bypass';

        return $this->sendRequest( $url, $booking_data );
    }

    /**
     * @param $name
     * @return string
     */
    private function getEndpointUrl($name)
    {
        return $this->host . $this->endpoints[$name];
    }

    /**
     * @param $url
     * @param $data
     * @return array|WP_Error
     */
    private function sendRequest($url, $data)
    {
//        error_log(json_encode($data), 0);
        return \wp_remote_post($url,  [
            'body' => $data
        ]);
    }
}