<?php

namespace JimsCustomerBooking;

use JimsCustomerBooking\Helpers\StringHelper;

if (!class_exists ('JimsCustomerBooking\Jims_Options')) {

    /**
     *
     */
    class Jims_Options
    {

        /**
         * @var default options
         */
        protected $options = [
            'http_schema' => 'https',
            'domain' => 'cb.jims.net',
            'DivisionId' => null,
            'bookingFormDivisionId' => null,
            'countryCode' => 'AU',
            'jimsProperty' => null,
            'primaryColor' => null,
            'textColor' => null,
            'logoUrl' => null,
            'gtmId' => null,
            'gatId' => 'Analytics ID',
            'cssSelector' => null,
            'redirectOnly' => "1"
        ];

        /**
         * @param $options
         */
        public function __construct($options = null)
        {
            if (is_array ($options) && $this->hasParams ($options)) {
                $this->options = $options;
            }
        }

        /**
         * @param $name
         * @param $value
         * @return void
         */
        public function addProperty($name, $value)
        {
            $this->options[$name] = $value;
        }

        /**
         * @return array
         */
        public function getProperties($filtered = true)
        {
            $options = $this->options;

            if ($filtered) {
                $options = $this->filterOptions ($options);
            }

            return apply_filters ( StringHelper::GetPrefixed ('customer_booking_options'), $options );
        }

        /**
         * @param string $name
         * @param array $arguments
         * @return null
         */
        public function __call(string $name, array $arguments)
        {
            return null;
        }

        public function __set($name, $value)
        {
            $this->options[$name] = $value;
        }

        /**
         * @param $name
         * @return null
         */
        public function __get($name)
        {
            return isset($options[$name]) ? $this->options[$name] : null;
        }

        /**
         * @param $option_arr
         * @return bool
         */
        protected function hasParams($option_arr)
        {
            $diff = array_diff (array_keys ($this->options, $option_arr));
            return count ($diff) == 0;
        }

        /**
         * Filters the options array
         * This fixes the bug with special characters in the cb url
         *
         * @param $options
         * @return mixed
         */
        protected function filterOptions($options)
        {
            //filter # from hex color
            $options['primaryColor'] = str_replace ('#', '', $options['primaryColor']);
            $options['textColor'] = str_replace ('#', '', $options['textColor']);
            return $options;
        }
    }
}