<?php

namespace JimsCustomerBooking;

use JimsCustomerBooking\Admin\BackendPages\CustomerBookingPage;
use JimsCustomerBooking\Admin\BackendPages\CustomerBookingSettingsPage;
use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Admin\BackendPages\FormCustomizeSettingsPage;
use JimsCustomerBooking\Database\Database;
use JimsCustomerBooking\Generators\CBURLGenerator;
use JimsCustomerBooking\Helpers\ColorHelper;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Admin\BackendPages\CBLinkGeneratorPage;
use JimsCustomerBooking\Admin\BackendPages\SettingsPage;
use JimsCustomerBooking\Admin\BackendPages\ShortCodeGeneratorPage;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Middlewares\Http;
use JimsCustomerBooking\RestfulAPI\AuthenticationController;
use JimsCustomerBooking\RestfulAPI\BookingsStatisticsController;
use JimsCustomerBooking\RestfulAPI\CustomerBookingController;
use JimsCustomerBooking\RestfulAPI\CustomerBookingCustomizeController;
use JimsCustomerBooking\RestfulAPI\InputValidationController;
use JimsCustomerBooking\RestfulAPI\OptionController;
use JimsCustomerBooking\RestfulAPI\ServiceController;
use JimsCustomerBooking\Services\DivisionService;
use JimsCustomerBooking\Shortcodes\BookingFormShortCode;
use JimsCustomerBooking\Shortcodes\CBFormShortCode;

if (!class_exists('JimsCustomerBooking\Jims_Plugin')) {
    /**
     *
     */
    class Jims_Plugin
    {
        /**
         * @var string
         */
        private $mode = 'development';


        public static $version = '3.3.02';

        /**
         * key for the option in wordpress options table
         */
        const OPTION_NAME = 'wpcb';


        public const SECKEY = 'jims-wpcb-communication-key-v1';

        /**
         * --- PLEASE DO NOT CHANGE THIS ---
         * menu slug for admin
         */
        public const PREFIX = 'jims_wpcb';


        public const FMS_API_KEY = '3e8bce9e-be45-4499-b31a-23b6fb067aa7';

        public const FMS_TOKENS = [
            'secret' => 'yw86Dvcwwsm0vEpZ3oNRFrXkCNovNLKh',
            'fso_token' => 'Zm1zX3VzZXI6Q2I2MkFZaFRwOFdtYUJSMnBzUmpSNUxTTnNhRWY1cEw=',
            'fse_token' => 'amltc19qb2JzX21vYmlsZTpDNHFkOFl0OGVnWVVLMnRaZnlhNWtaUjVZR3JmNng0eQ=='
        ];

        public const FINDAJIM_TOKEN = '84bcb06b2d534989cf811f250b26dc10';


        public const FMS_API_MODES =  [
            'production', 'uat', 'hf', 'hf2', 'int', 'development', 'local' //local is for testing and available only for local environments
        ];

        /**
         * @var array
         */
        private array $rest_classes = [];


        private $registered = [];

        /**
         * main entry point
         */
        public function run()
        {
            add_action('plugins_loaded', [$this, 'activate']);
//            add_action('init', [$this, 'OverrideShortCodes'], 1);
            add_action('init', [$this, 'SetupShortcodes']);
            add_action('wp_enqueue_scripts', [$this, 'frontScripts']);
            add_action('admin_enqueue_scripts', [$this, 'adminScripts']);
            add_action('rest_api_init', array($this, 'registerRestAPI'));
            add_action('wp_footer', [$this, 'setupCBModal']);
            add_action(StringHelper::GetPrefixed('jims_logo'), [$this, 'setupJimsLogo']);

            //filters
            add_filter( 'register_block_type_args', [$this, 'OverrideShortCodes'], 10, 2 );

            function foo_register_gallery() {
                register_block_type( 'core/gallery', array(
                    'render_callback' => 'foo_gallery_render',
                ) );
            }

            //Checks the theme and theme version and modifies settings to match
            $this->setupCompatibility();

            $this->registerBackendPages();

            $this->setupMiddleWares();
        }

        /**
         * Adds frontend scripts
         * @return void
         */
        public function frontScripts()
        {
            wp_register_script(StringHelper::GetPrefixed('js'), plugins_url('/assets/dist/js/front.js', JIMS_CBP_FILE), [], $this->getVersion(), true);

            wp_register_style(StringHelper::GetPrefixed('final'), false);

            list($main_color, $main_color_dim) = $this->getFormColors();
            wp_add_inline_style(StringHelper::GetPrefixed('final'), \sprintf(":root { --nwpcb-main-color: %s; --nwpcb-main-color-dim: %s }", $main_color, $main_color_dim));

            wp_enqueue_style(StringHelper::GetPrefixed('style'), plugins_url('/assets/dist/css/front.css', JIMS_CBP_FILE), [
                StringHelper::GetPrefixed('final')
            ], $this->getVersion());


            OptionHelper::localizeFormVariables('js');
            wp_enqueue_script(StringHelper::GetPrefixed('js'));
        }

        /**
         * @return void
         */
        public function adminScripts()
        {
            wp_enqueue_script(StringHelper::GetPrefixed('html-extension'), plugins_url('/assets/dist/js/html-extension.js', JIMS_CBP_FILE), [], $this->getVersion());

            wp_register_script(StringHelper::GetPrefixed('admin-js'), plugins_url('/assets/dist/js/admin.js', JIMS_CBP_FILE), [], $this->getVersion(), true);
            OptionHelper::localizeFormVariables('admin-js');

            wp_enqueue_script(StringHelper::GetPrefixed('admin-js'));
            wp_enqueue_style(StringHelper::GetPrefixed('admin-style'), plugins_url('/assets/dist/css/admin.css', JIMS_CBP_FILE), [], $this->getVersion());

            //enqueue fontawesome if not theme is not the new jims theme
            if (strpos(strtolower(wp_get_theme()->Name), 'jims') === false || strpos(strtolower(wp_get_theme()->Name), 'enfold') !== false) {
                wp_register_style(StringHelper::GetPrefixed('fontawesome-solid'), plugins_url('/assets/dist/fonts/css/solid.css', JIMS_CBP_FILE), [], $this->getVersion());
                wp_enqueue_style(StringHelper::GetPrefixed('fontawesome-style'), plugins_url('/assets/dist/fonts/css/fontawesome.css', JIMS_CBP_FILE), [StringHelper::GetPrefixed('fontawesome-solid')], $this->getVersion());
            }

        }

        /**
         * @return array
         */
        public function registerBackendPages()
        {
            return $this->registered['backend_pages'] = [
                SettingsPage::class => new SettingsPage(),
                NativeFormSettingsPage::class => new NativeFormSettingsPage(),
                FormCustomizeSettingsPage::class => new FormCustomizeSettingsPage(),
                CustomerBookingPage::class => new CustomerBookingPage(),
                CustomerBookingSettingsPage::class => new CustomerBookingSettingsPage(),
                CBLinkGeneratorPage::class => new CBLinkGeneratorPage(),
                ShortCodeGeneratorPage::class => new ShortCodeGeneratorPage(),
            ];
        }

        public function SetupShortcodes()
        {
            return $this->registered['shortcodes'] = [
                CBFormShortCode::class => new CBFormShortCode(),
                BookingFormShortCode::class => new BookingFormShortCode()
            ];
        }

        //$args, $name
        public function OverrideShortCodes($args, $name)
        {
//            register_block_type( 'core/shortcode', array(
//                'render_callback' => 'GutenbergBlockHelper::ShortCodeRender',
//            ) );

            if ( $name === 'core/shortcode' ) {
                $args['render_callback'] = 'GutenbergBlockHelper::ShortCodeRender';
            }
            return $args;
        }

        /**
         * @return void
         */
        public function registerRestAPI()
        {
            $this->rest_classes = [
                InputValidationController::class => new InputValidationController(),
                OptionController::class => new OptionController(),
                ServiceController::class => new ServiceController(),
                AuthenticationController::class => new AuthenticationController(),
                CustomerBookingController::class => new CustomerBookingController(),
                BookingsStatisticsController::class => new BookingsStatisticsController(),
                CustomerBookingCustomizeController::class => new CustomerBookingCustomizeController()
            ];

            //register rest api controllers hook
            $this->rest_classes = apply_filters(self::PREFIX . '_rest_controller_register', $this->rest_classes);
        }

        /**
         * @return Http[]
         */
        protected function setupMiddleWares(): array
        {
            return $this->registered['middlewares'] = [
                Http::class => new Http()
            ];
        }

        /**
         * @return void
         */
        public function setupCBModal()
        {
            $options = self::getOptions();

            (new Template('front'))->render([
                'options' => $options,
                'generator' => new CBURLGenerator($options->getProperties())
            ]);
        }

        /**
         * @return Jims_Options
         */
        public static function getOptions()
        {
            return OptionHelper::getEmbeddedFormOptions();
        }

        /**
         * @return void
         */
        public function activate()
        {
            $option_name = StringHelper::GetPrefixed('version', 'machine');
            $current_version = get_option($option_name, '0.0.0');

            if ($current_version != JIMS_CBP_VERSION) {
                update_option($option_name, JIMS_CBP_VERSION, true);
                Database::setupOrUpdateDatabase($current_version);

                //update website on leads.jims.net
                $this->updateWebsiteToLeads();
            }
        }

        public function getVersion()
        {
            if ($this->mode == 'development') return time();
            else return self::$version;
        }

        public function setupCompatibility()
        {
            $theme = wp_get_theme();
            $theme_name = $theme->get('Name');
            $theme_version = $theme->get('Version');

            // check if enfold theme is active and get logo from theme options
            add_filter(StringHelper::getPrefixed('booking_form_logo_url'), function ($logo) use ($theme_name, $theme_version) {

                if (str_contains(strtolower($theme_name), 'enfold')) {
                    $options = get_option('avia_options_jims_enfold');
                    $logo = isset($options['avia'], $options['avia']['logo']) ? $options['avia']['logo'] : '';
                }

                return $logo;
            }, 99, 1);
        }

        /**
         * @param $name
         * @return mixed
         */
        public function getRegistered($name): array
        {
            return isset($this->registered[$name]) ? $this->registered[$name] : $this->registered;
        }

        public function setupJimsLogo()
        {
            $plugin_url = plugins_url('/assets/dist/img/jims.png', JIMS_CBP_FILE);
            printf('<img src="%s" alt="%s">', $plugin_url, "Jims Logo");
        }

        /**
         * @return array
         */
        public function getFormColors(): array
        {
            $main_color = OptionHelper::getValue(NativeFormSettingsPage::$option_name, 'bookingFormMainColor', '');
            if (empty($main_color)) {
                $main_color = '#0b75b4';
            }

            $main_color_dim = \sprintf('rgb(%s,%s,%s)', ...ColorHelper::darken($main_color, 10, false));
            return array($main_color, $main_color_dim);
        }

        public function updateWebsiteToLeads()
        {
            //check if division id is set
            $division_id = OptionHelper::getFormOption('bookingFormDivisionId');
            $country = OptionHelper::getFormOption('country');
            $debug_mode = OptionHelper::getFormOption('debugMode', 0);

            error_log('update leads', 0);
            error_log($division_id, 0);

            if (!empty($division_id)) {
                $integration = new Integrations\JimsLeadsIntegration();
                return $integration->updateSite($division_id, $country, $debug_mode);
            }
        }
    }
}