<?php

namespace JimsCustomerBooking\Middlewares;

use JimsCustomerBooking\Generators\CBURLGenerator;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Jims_Options;

if( ! class_exists('JimsCustomerBooking\Middlewares\Http')){
    /**
     * Allows CB pages (cb.jims.net, cb20.jims.net) to exists as iframes in the site
     * and allows the site to communicate with the iframe
     */
    class Http {

        private $options = null;

        /**
         * @var string
         */
        private $domain = 'https://cb.jims.net';
        /**
         *
         */
        public function __construct()
        {
            $this->getOptions();

            /**
             * @var Jims_Options $options
             */
            $this->domain = "{$this->options->http_schema}://{$this->options->domain}";

            //add_action('init',[$this, 'ModifyHeaders']);
            add_filter('wp_headers', [$this, 'ModifyHeaders']);
            add_filter( 'allowed_http_origins', [$this, 'AddAllowedOrigins'] );
        }

        /**
         * @param $headers
         * @return array
         */
        public function ModifyHeaders($headers){

            $headers['Access-Control-Allow-Origin'] = "*";
            $headers['Access-Control-Allow-Methods'] = 'GET, POST, OPTIONS';
//            $headers['X-Frame-Options'] = "allow-from https://cb20.jims.net";
            $headers['X-Frame-Options'] = "allow-from {$this->domain}";
//            $headers['Content-Security-Policy'] = "frame-ancestors 'self' {$this->domain}; frame-src 'self' {$this->domain};";
            $headers['Access-Control-Allow-Headers'] = 'Content-Type';
            $headers['Cross-Origin-Opener-Policy'] = "same-origin-allow-popups";
            $headers['Cross-Origin-Resource-Policy'] = "cross-origin";

            return $headers;
        }

        /**
         * @param $origins
         * @return array
         */
        public function AddAllowedOrigins($origins){
//            $origins[] = 'https://cb20-int.cloud.jims.net/';
            $origins[] = $this->domain.'/';
//            $origins[] =  'https://internal.jims.net/';
//            $origins[] = 'http://localhost:8089/';
            return $origins;
        }

        public function getOptions(){
            $options = OptionHelper::get('form', new Jims_Options());

            if( !($options instanceof Jims_Options) ){
                $options = unserialize($options);
            }

            $this->options = $options;
        }
    }
}