<?php
/**
 * Created by Konstantinos Tsatsarounos.
 * Date: 4/8/2019
 * Time: 10:55 AM
 */

namespace JimsCustomerBooking\RestfulAPI;

use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\RestfulAPI\RestController;


if( !class_exists('JimsCustomerBooking\RestfulAPI\CustomerBookingCustomizeController') ){
    /**
     *
     */
    class CustomerBookingCustomizeController extends RestController
    {
        protected $route = 'customize';

        public function register_routes(){

            register_rest_route(
                $this->get_namespace(),
                $this->get_route() . '/service-order',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => array($this, 'update_service_order'),
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => array($this, 'get_service_order'),
                        'permission_callback' => '__return_true',
                    ]
                ]
            );

            register_rest_route(
                $this->get_namespace(),
                $this->get_route() . '/service-groups',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => array($this, 'update_service_groups'),
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => array($this, 'get_service_groups'),
                        'permission_callback' => '__return_true',
                    ]
                ]
            );

            register_rest_route(
                $this->get_namespace(),
                $this->get_route() . '/service-list',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => array($this, 'update_service_list'),
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => array($this, 'get_service_list'),
                        'permission_callback' => '__return_true',
                    ]
                ]
            );

            register_rest_route(
                $this->get_namespace(),
                $this->get_route() . '/service-descriptions',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => array($this, 'update_service_descriptions'),
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => array($this, 'get_service_descriptions'),
                        'permission_callback' => '__return_true',
                    ]
                ]
            );

            register_rest_route(
                $this->get_namespace(),
                $this->get_route() . '/options',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => array($this, 'update_options'),
                        'permission_callback' => '__return_true',
                    ],
                    [
                        'methods' => \WP_REST_Server::READABLE,
                        'callback' => array($this, 'get_options'),
                        'permission_callback' => '__return_true',
                    ]
                ]
            );

            register_rest_route(
                $this->get_namespace(),
                $this->get_route() . '/delete-all-options',
                [
                    [
                        'methods' => \WP_REST_Server::CREATABLE,
                        'callback' => array($this, 'delete_all_options'),
                        'permission_callback' => '__return_true',
                    ]
                ]
            );

        }

        /**
         * It saves the service order as an array of objects in the database
         * every object has the service id and the gravity number.
         *
         * The gravity number defines the order of the service in the booking form
         *
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function update_service_order(\WP_REST_Request $request){
            list($result, $data) = $this->saveOption($request, 'customer_booking_service_order');

            return $this->response([
                'result' => $result ? 'success' : 'failure',
                'data' => isset($data) ? $data : []
            ], 200);
        }

        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function get_service_order(\WP_REST_Request $request){
            $option_name = StringHelper::GetPrefixed('customer_booking_service_order', 'machine');
            $data = get_option($option_name, []);

            return $this->response([
                'result' => !empty($data) ? 'success' : 'failure',
                'data' => $data
            ], 200);
        }

        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function update_service_groups(\WP_REST_Request $request){
            list($result, $data) = $this->saveOption($request, 'customer_booking_service_groups');

            return $this->response([
                'result' => $result ? 'success' : 'failure',
                'data' => isset($data) ? $data : []
            ], 200);
        }

        /**
         * Returns the service ids in a grouped manner, with descriptions
         *
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function get_service_groups(\WP_REST_Request $request){
            $option_name = StringHelper::GetPrefixed('customer_booking_service_groups', 'machine');
            $data = get_option($option_name, []);

            return $this->response([
                'result' => !empty($data) ? 'success' : 'failure',
                'data' => $data
            ], 200);
        }


        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function update_service_list(\WP_REST_Request $request){
            error_log(json_encode($request->get_body()), 0);
            return $this->response(['data' => $request->get_body()], 200);
        }

        /**
         * Service List is a list of preselected services, it's like a package
         *
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function get_service_list(\WP_REST_Request $request){
            error_log(json_encode($request->get_body()), 0);
            return $this->response(['data' => $request->get_body()], 200);
        }

        public function update_service_descriptions(\WP_REST_Request $request){
            list($result, $data) = $this->saveOption($request, 'customer_booking_service_descriptions');

            return $this->response([
                'result' => $result ? 'success' : 'failure',
                'data' => isset($data) ? $data : []
            ], 200);
        }

        public function get_service_descriptions(\WP_REST_Request $request){
            $option_name = StringHelper::GetPrefixed('customer_booking_service_descriptions', 'machine');
            $data = get_option($option_name, []);

            return $this->response([
                'result' => !empty($data) ? 'success' : 'failure',
                'data' => $data
            ], 200);
        }

        public function update_options(\WP_REST_Request $request){
            list($result, $data) = $this->saveOption($request, 'customer_booking_customizer_options');

            return $this->response([
                'result' => $result ? 'success' : 'failure',
                'data' => isset($data) ? $data : []
            ], 200);
        }

        public function get_options(\WP_REST_Request $request){
            $option_name = StringHelper::GetPrefixed('customer_booking_customizer_options', 'machine');
            $data = get_option($option_name, []);

            return $this->response([
                'result' => !empty($data) ? 'success' : 'failure',
                'data' => $data
            ], 200);
        }

        /**
         * @param \WP_REST_Request $request
         * @return array
         */
        public function saveOption(\WP_REST_Request $request, $optionName): array
        {
            $body = $request->get_body();
            $option_name = StringHelper::GetPrefixed($optionName, 'machine');
            $result = false;

            if (StringHelper::isJson($body)) {
                $data = json_decode($body, true);

                if (!empty($data)) {
                    $result = update_option($option_name, $data);
                }
            }
            return array($result, $data);
        }

        public function delete_all_options(\WP_REST_Request $request){
            $option_name = StringHelper::GetPrefixed('customer_booking_service_order', 'machine');
            delete_option($option_name);

            $option_name = StringHelper::GetPrefixed('customer_booking_service_groups', 'machine');
            delete_option($option_name);

            $option_name = StringHelper::GetPrefixed('customer_booking_service_descriptions', 'machine');
            delete_option($option_name);

            $option_name = StringHelper::GetPrefixed('customer_booking_customizer_options', 'machine');
            delete_option($option_name);

            return $this->response([
                'result' => 'success',
                'data' => []
            ], 200);
        }
    }
}


