<?php

namespace JimsCustomerBooking\RestfulAPI;

use libphonenumber\PhoneNumberUtil as libPhoneNumberUtil;
use \libphonenumber\geocoding\PhoneNumberOfflineGeocoder as libPhoneNumberOfflineGeocoder;

class InputValidationController extends RestController
{
    protected $route = 'input-validation';

    public function register_routes()
    {
        register_rest_route( $this->get_namespace(), '/' . $this->route . '/phone', [
            [
                'methods' => \WP_REST_Server::READABLE,
                'callback' => [$this, 'validate_phone'],
                'permission_callback' => '__return_true',
            ]
        ]);
    }

    /**
     * @param \WP_REST_Request $request
     * @return \WP_REST_Response
     */
    public function validate_phone(\WP_REST_Request $request){
        $phone = $request->get_param('number');
        $country = $request->get_param('country');
        $isValid = false;
        $data = [
            'phone' => $phone,
            'country' => $country,
        ];

        try {
            $phoneUtil = libPhoneNumberUtil::getInstance();
            $phoneNumber = $phoneUtil->parse($phone, $country);
            $geocoder =  libPhoneNumberOfflineGeocoder::getInstance();

            $isValid = $phoneUtil->isValidNumber($phoneNumber);
            $data['description'] = $geocoder->getDescriptionForNumber($phoneNumber, 'en_US');
        } catch (\libphonenumber\NumberParseException $e) {
            error_log($e->getMessage ());
        }

        return rest_ensure_response([
            'result' => $isValid,
            'data' => $data,
        ]);
    }


}