<?php

namespace JimsCustomerBooking\RestfulAPI;


use JimsCustomerBooking\Admin\BackendPages\NativeFormSettingsPage;
use JimsCustomerBooking\Database\Database;
use JimsCustomerBooking\Helpers\OptionHelper;
use JimsCustomerBooking\Helpers\StringHelper;
use JimsCustomerBooking\Integrations\JimsLeadsIntegration;
use JimsCustomerBooking\Jims_Options;

if ( ! class_exists('JimsCustomerBooking\RestfulPI\OptionController')) {
    /**
     *
     */
    class OptionController extends RestController
    {
        /**
         * @var string
         */
        protected $route = 'options';

        /**
         * @return void
         */
        public function register_routes(){

            register_rest_route($this->get_namespace(), $this->get_route() . '/form', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => array($this, 'update_form'),
                    'permission_callback' => '__return_true',
                ],
                [
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_form'),
                    'permission_callback' => '__return_true',
                ]
            ]);

            register_rest_route($this->get_namespace(), $this->get_route() . '/form_admin', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => array($this, 'delete_form'),
                    'permission_callback' => '__return_true',
                ],
                [
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => array($this, 'refresh_form'),
                    'permission_callback' => '__return_true',
                ]
            ]);

            register_rest_route($this->get_namespace(), $this->get_route() . '/form-events', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => array($this, 'update_events_form'),
                    'permission_callback' => '__return_true',
                ],
                [
                    'methods' => \WP_REST_Server::READABLE,
                    'callback' => array($this, 'get_events_form'),
                    'permission_callback' => '__return_true',
                ]
            ]);

            register_rest_route($this->get_namespace(), $this->get_route() . '/clear-transients', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => array($this, 'clear_transients'),
                    'permission_callback' => '__return_true',
                ]
            ]);

            register_rest_route($this->get_namespace(), $this->get_route() . '/leads', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => array($this, 'update_leads_website'),
                    'permission_callback' => '__return_true',
                ]
            ]);

            register_rest_route ($this->get_namespace (), $this->get_route () . '/form-events-admin', [
                [
                    'methods' => \WP_REST_Server::CREATABLE,
                    'callback' => array($this, 'delete_events_form'),
                    'permission_callback' => '__return_true',
                ]
            ]);
        }

        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function update_form($request){
            $options = new Jims_Options( $request->get_params() );
            $result = update_option( StringHelper::GetPrefixed('form', 'machine'), $options );

            if($options->bookingFormDivisionId != null){
                try {
                    $integration = new JimsLeadsIntegration();
                    $debug_mode = property_exists($options, 'debugMode') ? $options->debugMode : 0;

                    $integration->updateSite($options->bookingFormDivisionId, $options->countryCode, $debug_mode);
                } catch (\Exception $e) {
                    error_log($e->getMessage());
                }
            }

            return $this->response([
                'status' => $result ? 200 : 500,
                'message' => $result ? 'success' : 'failure'
            ]);
        }

        /**
         * @param $request
         * @return \WP_REST_Response
         */
        public function get_form($request){
            $data = $this->get_form_data ();
            return $this->response( $data->getProperties() );
        }

        public function get_form_data(){
            $form  = OptionHelper::get('form', [] );

            /**
             * @var Jims_Options $data
             */
            $formOptions = OptionHelper::get('form', [] );

            return is_string($form) ? unserialize( $formOptions, [
                'allowed_classes' => true
            ] ) : $form;
        }


        /**
         * @param $request
         * @return \WP_REST_Response
         */
        public function refresh_form($request){
            /**
             * @var Jims_Options $options
             */
            $options = $this->get_form_data ();

            $result = delete_option(StringHelper::GetPrefixed('form', 'machine'));
            //$result = update_option( StringHelper::GetPrefixed('form', 'machine'), $options );
            //var_dump ($options->getProperties (), $result);

            return $this->response([
                'status' => $result ? 200 : 500,
                'message' => $result ? 'success' : 'failure',
                'fields' => $options->getProperties ()
            ]);
        }

        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function delete_form($request){
            $params = $request->get_params();

            $result = false;
            if( isset( $params['allow'] ) && $params['allow'] == 'admin_'.md5('admin') ) {
                $result = delete_option(StringHelper::GetPrefixed('form', 'machine'));
            }
            return $this->response([
                'status' => $result ? 200 : 500,
                'message' => $result ? 'success' : 'failure'
            ]);
        }

        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function clear_transients(\WP_REST_Request $request){
            $count = Database::deleteAllTransients();

            return $this->response([
                'message' => $count ? 'success' : 'failure',
                'count' => $count
            ], $count ? 200 : 500);
        }

        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function update_leads_website(\WP_REST_Request $request){
            $integration = new JimsLeadsIntegration();

            $division_id = OptionHelper::getFormOption('bookingFormDivisionId');
            $country = OptionHelper::getFormOption('country');
            $debug_mode = OptionHelper::getFormOption('debugMode', 0);

            $integration->updateSite($division_id, $country, $debug_mode);

            return $this->response([
                'message' => 'success'
            ], 200);
        }

        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function update_events_form($request){
            $options = $request->get_params();
            $result = update_option( StringHelper::GetPrefixed( NativeFormSettingsPage::$option_name, 'machine'), $options );

            return $this->response([
                'data' => $options,
                'status' => $result ? 200 : 500,
                'message' => $result ? 'success' : 'failure'
            ]);
        }

        public function delete_events_form($request){
            $params = $request->get_params();

            $result = false;
            if( isset( $params['allow'] ) && $params['allow'] == 'admin_'.md5('admin') ) {
                $result = delete_option(StringHelper::GetPrefixed( NativeFormSettingsPage::$option_name, 'machine'));
            }
            return $this->response([
                'status' => $result ? 200 : 500,
                'message' => $result ? 'success' : 'failure'
            ]);
        }

        /**
         * @param \WP_REST_Request $request
         * @return \WP_REST_Response
         */
        public function get_events_form($request){
            $form  = OptionHelper::get(NativeFormSettingsPage::$option_name, [] );
            return $this->response($form);
        }

    }
}