<?php

namespace JimsCustomerBooking\Services;

use JimsCustomerBooking\Helpers\StringHelper;

class TokenManager extends Service
{
    static function encode($key, $token, $duration = '1 hour'){
        return base64_encode(self::encrypt($key, $token, $duration));
    }

    static function decode($key, $token){
        return self::decrypt($key, base64_decode($token));
    }

    /**
     * Encrypt the token using the key
     *
     * @param $key
     * @param $token
     * @return string
     */
    static function encrypt($key, $token, $duration = '1 hour'){
        $key = self::getScrampledKey($key);

        $json_token = json_encode([
            'token' => substr($token, 7) . substr($token, 0, 7),
            'created_at' => date('Y-m-d H:i:s'),
            'duration' => $duration
        ]);

        // reverse the token
        $reversed_token = strrev($json_token);

        //use the key, if a character exists on the key, replace the character on the token with the next character on the key
        return $key . '-' . $reversed_token;
    }

    /**
     * Decrypt the token using the key
     *
     * @param $key
     * @param $token
     * @return string
     */
    static function decrypt($key, $encrypted_token){
        //replace single dashes, not replace --
        $content = self::getContent($key, $encrypted_token);

        if(!empty($content)){
            $json = json_decode($content, true);

            //get first value of the array
            $token = isset($json['token']) ? $json['token'] : '';

            return self::shiftTokenBack($token);
        }

        //shift the token by 7 characters
        return '';
    }

    static function getContent($key, $encrypted_token){
        //replace single dashes, not replace --
        $encrypted_token = str_replace( self::getScrampledKey($key) . '-', '', $encrypted_token);

        //reverse the token
        $shifted_token = strrev($encrypted_token);

        if(StringHelper::isJson($shifted_token)){
            return $shifted_token;
        }

        //shift the token by 7 characters
        return '';
    }

    static function verify($key, $token){
        $content = self::getContent($key, base64_decode($token));

        if (!empty($content)){
            $json = json_decode($content, true);
            $token = isset($json['token']) ? self::shiftTokenBack($json['token']) : '';
            $is_valid_token = str_contains($token, 'wpcb');


            //get first value of the array
            $created_at = isset($json['created_at']) ? $json['created_at'] : '';

            //get first value of the array
            $duration = isset($json['duration']) ? $json['duration'] : '';

            if($duration == 'forever' && $is_valid_token) return true;

            list($diff_in_seconds, $duration_in_seconds) = self::calculatedTokenTime($created_at, $duration);

            return $diff_in_seconds <= $duration_in_seconds && $is_valid_token;
        }
    }

    static function getScrampledKey($key){
        $key = implode('', array_unique(str_split($key)));

        $key = preg_replace('/[^a-zA-Z0-9]/', '', $key);

        return $key;
    }

    static function shiftTokenBack($token){
        return !empty($token) ? substr($token, -7) . substr($token, 0, -7) : '';
    }

    /**
     * @param mixed $created_at
     * @param mixed $duration
     * @return array
     * @throws \Exception
     */
    public static function calculatedTokenTime(mixed $created_at, mixed $duration): array
    {
        $now = new \DateTime();

        $created_at = new \DateTime($created_at);

        $diff = $now->diff($created_at);

        $diff_in_seconds = $diff->days * 24 * 60 * 60;
        $diff_in_seconds += $diff->h * 60 * 60;
        $diff_in_seconds += $diff->i * 60;
        $diff_in_seconds += $diff->s;

        $duration_in_seconds = strtotime($duration) - strtotime('today');
        return array($diff_in_seconds, $duration_in_seconds);
    }

    public static function generateRandomString(int $int)
    {
        $characters = '0123456789abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ';
        $charactersLength = strlen($characters);
        $randomString = '';

        for ($i = 0; $i < $int; $i++) {
            $randomString .= $characters[rand(0, $charactersLength - 1)];
        }

        return $randomString;
    }
}