<?php

namespace JimsCustomerBooking;

use JimsCustomerBooking\Helpers\StringHelper;

if( ! class_exists('JimsCustomerBooking\Template')){
    /**
     *
     */
    class Template {
        /**
         * @var string
         */
        protected $base_folder = JIMS_CBP_DIR . '/includes/templates/';

        /**
         * @var string
         */
        protected $subfolder = '';

        /**
         * @var string
         */
        protected $default_template = 'default';

        /**
         * @var string
         */
        protected $suffix = 'tpl.php';

        /**
         * @var string
         */
        protected $name = '';

        /**
         * @param string $template_name
         */
        public function __construct(string $template_name)
        {
            if($this->TemplateExists($template_name)){
                $this->name = $template_name;
            } else {
                $this->name = $this->default_template;
            }
            return $this;
        }

        /**
         * @param string $template_name
         */
        public function TemplateExists(string $template_name){
            return \file_exists( $this->GetTemplatePath($template_name) );
        }

        /**
         * @param string $template_name
         * @return string
         */
        public function GetTemplatePath(string $template_name){
            return sprintf('%s%s%s.%s',$this->base_folder , $this->subfolder , $template_name , $this->suffix);
        }

        /**
         * @param array $data
         */
        public function render(array $data = [], $with_placeholders = false, $suffix_html = ''){
            extract($data);
            $closing_html = '';

            if(!empty($suffix_html)){
                $closing_html = $suffix_html;
            }

            if($with_placeholders){
                $html = $this->get($data);
                echo $html . $closing_html;
                return;
            }

            include $this->GetTemplatePath( $this->name );
            echo $closing_html;
        }

        /**
         * @param array $data
         */
        public function get(array $data = []){
            extract($data);
            ob_start();
            include $this->GetTemplatePath( $this->name );
            $output = ob_get_clean();

            $html = apply_filters ( StringHelper::GetPrefixed ( 'template_output', 'machine' ), $output, $this->name, $data );

            return $this->ReplacePlaceholders ( $html, $data );
        }

        /**
         * @param $html
         * @param $data
         * @return array|string|string[]
         */
        protected function ReplacePlaceholders($html, $data){
            //remove arrays from data
            $data = array_filter($data, function($value){
                return ! is_array($value) && ! is_object($value);
            });

            $placeholders = array_map(function($key) use ($data){
                return sprintf('{{%s}}', $key);
            }, array_keys($data) );

            return str_replace($placeholders, array_values($data), $html);
        }

        public function getName(): string
        {
            return $this->name;
        }
    }
}