<?php
/**
 * Template Name: Customer Bookings List
 *
 * @package JimsCustomerBooking
 * @var array $statistics
 * @var $pages     array
 * @var $parts     array
 */

$sended = $statistics['sended'];
$today = $statistics['today'];


/**
 * @var $header \JimsCustomerBooking\Template
 * @var $footer \JimsCustomerBooking\Template
 */
$header = $footer = null;
extract ($parts);

$failed_bookings = [];
$success_bookings = [];

foreach ($sended as $key => $value) {
    if(!isset($value['title'])) continue;

    $failed_bookings[$value['title']] = (int) $value['total_bookings'] - (int) $value['total_bookings_with_status'];
    $success_bookings[$value['title']] = (int) $value['total_bookings_with_status'];
}
?>
<script>
    window.graphData = <?php echo json_encode([
        'failed' => $failed_bookings,
        'success' => $success_bookings
    ]); ?>
</script>
<style>
    @import url('https://fonts.googleapis.com/css2?family=Roboto:wght@400;500;700&display=swap');

    .jims-wpcb-wrapper section.jims-main {
        max-width: 100%;
        min-height: 100vh;
    }

    .dashboard-panel {
        max-width: 1280px;
        font-family: 'Roboto', sans-serif;
        background-color: #fff;
        /*padding: 20px;*/
        margin-bottom: 40px;
    }

    .panel-title {
        color: #0657a5;
        margin-bottom: 20px;
        font-weight: 600;
        /*text-transform: uppercase;*/
        /*letter-spacing: 0.05em;*/
    }

    .statistics {
        display:block;
    }

    .statistic {
        width: 100%;
        margin: 10px;
        padding: 20px;
        background-color: #f3f3f3;
        border-radius: 4px;
        box-shadow: 0 1px 1px rgba(0,0,0,0.1), 0 1px 2px rgba(0,0,0,0.1);
        animation: fadeIn 1s;
    }

    @keyframes fadeIn {
        from { opacity: 0; }
        to   { opacity: 1; }
    }

    .statistic-title {
        color: #3f51b5;
        margin-bottom: 10px;
        text-transform: uppercase;
        letter-spacing: 0.05em;
    }

    .statistic-data {
        color: #212121;
        font-size: 1.2em;
        margin-bottom: 10px;
    }

    .graphics {
        display: flex;
        flex-wrap: wrap;
        justify-content: flex-start;
    }
</style>


<?php if (!empty($header)): ?>
    <?php $header->render ([
        'title' => 'Customer Bookings',
        'description' => 'This page allows you to view all Customer Bookings and Statistics',
        'pages' => $pages,
        'url' => get_template_directory_uri()
    ], true); ?>
<?php endif; ?>



<div class="{{css_class}}">
    <section class="jims-main">
        <div class="dashboard-panel">
            <h3 class="panel-title">Overview</h3>

            <div class="graphics">
                <!-- Chart.js placeholder -->
                <div>
                    <canvas width="600px" height="600px" id="booking-overview"></canvas>
                </div>
            </div>
        </div>

        <!-- Table with bookings: id, title, status, created_at -->
        <div class="dashboard-panel">
            <h3 class="panel-title">Total Bookings</h3>

            <?php if (empty($sended)): ?>
                <p>No bookings found.</p>
            <?php else: ?>

                <div class="statistics">

                    <?php foreach ($sended as $index => $statistic): ?>
                        <div class="statistic">
                            <h3 class="statistic-title"><?php echo $statistic['title']; ?></h3>
                            <p class="statistic-data">Total Bookings: <?php echo $statistic['total_bookings'] ?></p>
                            <p class="statistic-data">Successful Bookings: <?php echo $statistic['total_bookings_with_status'] ?></p>
                        </div>
                    <?php endforeach; ?>
                </div>
            <?php endif; ?>
            <!-- Add more statistic divs for other data -->
        </div>

        <div class="dashboard-panel">
            <h3 class="panel-title">Today's Bookings</h3>

            <?php if (empty($today)): ?>
                <p>No bookings found.</p>
            <?php else: ?>
                <div class="statistics">

                    <?php foreach ($today as $index => $statistic): ?>
                        <div class="statistic">
                            <h3 class="statistic-title"><?php echo $statistic['title']; ?></h3>
                            <p class="statistic-data">Total Bookings: <?php echo $statistic['total_bookings'] ?></p>
                            <p class="statistic-data">Successful Bookings: <?php echo $statistic['total_bookings_with_status'] ?></p>
                        </div>
                    <?php endforeach; ?>
                </div>
            <?php endif; ?>
        </div>
    </section>
</div>
</div>
</div>
</div>