<?php
/**
 * @var $css_class string
 * @var $title     string
 * @var $options   array
 * @var $pages     array
 * @var $parts     array
 */

$service = new \JimsCustomerBooking\Services\DivisionService();
$divisions = $service->getDivisions();
$countries = $service->getCountries();

$fms_enviroment = [
    'production' => 'Production',
    'development' => 'Development',
    'uat' => 'UAT',
    'hf' => 'Hotfix',
    'int' => 'Integration'
//    'hf2' => 'Hotfix2',
];

$hostname = $_SERVER['HTTP_HOST'];
if(is_admin() && in_array($hostname, ['localhost', 'jims-wp-default.local', '127.0.0.1'])) {
    $fms_enviroment['local'] = 'Local';
}

/**
 * @var $header \JimsCustomerBooking\Template
 * @var $footer \JimsCustomerBooking\Template
 */
$header = $footer = null;

$header = $parts['header'];
$footer = $parts['footer'];
//extract ($parts);

$info_fields = [
    'firstname',
    'lastname',
    'email',
    'phone',
    'businessName',
    'comments'
];


$default_required_info_fields = array_filter($info_fields, function ($field) use ($options) {
    $default_required_info_fields = [
        'firstname',
        'lastname',
        'email',
        'phone',
    ];

    $in_database = isset($options['required_fields-' . \JimsCustomerBooking\Helpers\StringHelper::slugify($field)]);
    $in_default = in_array($field, $default_required_info_fields) && !$in_database;

    return $in_database || $in_default;
});

?>
<script>
    window.wpcbSecret = '<?php echo 'admin_' . md5('admin'); ?>';
</script>

<?php if (!empty($header)): ?>
    <?php $header->render([
        'title' => 'Settings',
        'description' => 'This page, allows to modify the behavior of Jim\'s customer booking form (native)',
        'pages' => $pages,
        'url' => get_template_directory_uri()
    ], true); ?>
<?php endif; ?>

<div class="<?php echo $css_class ?>">
    <section class="jims-main">
        <form id="jims-events-form">
            <div class="settings-text settings-text--blue">
                <ul>
                    <li>If this page doesn't work, please contact support and ask to whitelist your server's IP.
                    </li>
                </ul>
            </div>

            <div class="setting-group basic">
                <h3>Basic Settings</h3>
                <div class="inner">
                    <label>Enable CB3 Form</label>
                    <select name="cb3FormEnabled" id="enable-cb3-form" class="widefat">
                        <option value="1" <?php echo isset($options['cb3FormEnabled']) && $options['cb3FormEnabled'] == 1 ? 'selected' : ''; ?>>Yes</option>
                        <option value="0" <?php echo isset($options['cb3FormEnabled']) && $options['cb3FormEnabled'] == 0 ? 'selected' : ''; ?>>No</option>
                    </select>

                    <label for="divisions">Choose a Division</label>
                    <select id="divisions" name="bookingFormDivisionId" class="widefat">
                        <?php if (count($divisions) > 0): ?>
                            <?php foreach ($divisions as $division): ?>
                                <?php $selected = $division['id'] == $options['bookingFormDivisionId'] ? 'selected' : '' ?>
                                <option value="<?php echo $division['id']; ?>" <?php echo $selected ?>><?php echo $division['name']; ?></option>
                            <?php endforeach; ?>
                        <?php else: ?>
                            <option value="0">No Divisions Available</option>
                        <?php endif; ?>
                    </select>
                    <small>If this option doesn't work, please contact infotech@jims.net and ask to whitelist your
                        server's IP</small>

                    <label for="country">Country</label>
                    <select class="widefat" name="country" id="country">
                        <?php foreach ($countries as $country): ?>
                            <?php $selected = $country->symbol == $options['country'] ? 'selected' : '' ?>
                            <option value="<?php echo $country->symbol; ?>" <?php echo $selected ?> ><?php echo $country->name; ?></option>
                        <?php endforeach; ?>
                    </select>

                    <label for="phone">Phone</label>
                    <input type="text" class="widefat" id="phone" name="phone"
                           value="<?php echo isset($options['phone']) ? $options['phone'] : '' ?>">

                    <label for="debugMode">Enable Debug Mode</label>
                    <select name="debugMode" id="debug_mode" class="widefat">
                        <option value="0" <?php echo $options['debugMode'] == 0 ? 'selected' : ''; ?>>No</option>
                        <option value="1" <?php echo $options['debugMode'] == 1 ? 'selected' : ''; ?>>Yes</option>
                    </select>
                    <small>Enable it if this site is staging.</small>
                </div>
            </div>

            <!-- Settings group for styling -->
            <div class="setting-group style">
                <br/><br/>
                <hr class="gradient-hr">
                <h3>Styling</h3>
                <div class="inner">
                    <label for="css">Form Main Color</label>
                    <input name="bookingFormMainColor" type="color"
                           value="<?php echo isset($options['bookingFormMainColor']) ? $options['bookingFormMainColor'] : '#0b75b4' ?>">
                </div>
            </div>

            <div class="setting-group form-text <?php echo is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden' ?>">
                <br/><br/>
                <hr class="gradient-hr">
                <!-- Form Text Settings -->
                <h3>Form Labels</h3>
                <div class="inner">
                    <div class="settings-text settings-text-warning">
                        Please, to avoid TOS violation, before any changes in this section, notify the <a
                                href="mailto:infotech@jims.net">Infotech</a>.
                    </div>
                    <label for="form-title">Address Text</label>
                    <input type="text" class="widefat" id="form-address-text" name="bookingFormAddressText"
                           value="<?php echo isset($options['bookingFormAddressText']) ? $options['bookingFormAddressText'] : '' ?>">

                    <label for="form-subtitle">Details Text</label>
                    <input type="text" class="widefat" id="form-details-text" name="bookingFormDetailsText"
                           value="<?php echo isset($options['bookingFormDetailsText']) ? $options['bookingFormDetailsText'] : '' ?>">

                    <label for="form-button">Description Text</label>
                    <input type="text" class="widefat" id="form-button" name="bookingFormDescriptionText"
                           value="<?php echo isset($options['bookingFormDescriptionText']) ? $options['bookingFormDescriptionText'] : '' ?>">
                </div>
            </div>

            <div class="setting-group form-required-fields <?php echo is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden' ?>">
                <br/><br/>
                <hr class="gradient-hr">
                <!-- Form Text Settings -->
                <h3>Form Required Fields</h3>
                <div class="inner">
                    <div class="settings-text settings-text-warning">
                        Please, to avoid TOS violation, before any changes in this section, notify the <a
                                href="mailto:infotech@jims.net">Infotech</a>.
                    </div>

                    <!-- Multiselect for required fields -->
                    <label for="required-fields">Required Fields</label>
                    <?php foreach ($info_fields as $field): $cf = \JimsCustomerBooking\Helpers\StringHelper::slugify($field); ?>
                        <p class="checkbox-field">
                            <label style="display: inline" for="<?php echo $cf; ?>"><?php echo ucwords($field); ?></label>
                            <input type="checkbox" id="<?php echo $cf; ?>"  name="required_fields-<?php echo $cf ?>"
                                   value="<?php echo $field ?>" <?php echo in_array($field, $default_required_info_fields) ? 'checked': ''?>>
                        </p>
                    <?php endforeach; ?>
                </div>
            </div>


            <div class="setting-group auto-complete">
                <br/><br/>
                <hr class="gradient-hr">
                <!-- Google Auto Complete Settings-->
                <h3>Auto Complete Settings</h3>
                <div class="inner">
                    <div class="settings-text">The WPCB plugin offers two options for address autocomplete:
                        <ul>
                            <li>Use Jim's address autocomplete service: This option is free of charge but may have
                                slower
                                performance compared to Google Autocomplete (default setting).
                            </li>
                            <li>Use Google address autocomplete: This option provides superior performance, but requires
                                adding
                                a
                                credit card to obtain an API key.
                            </li>
                        </ul>
                    </div>

                    <!-- Select if the user wants to use Google Auto Complete -->
                    <label for="use-google-autocomplete">Choose Address Autocomplete service</label>
                    <select name="use_google_autocomplete" id="use-google-autocomplete" class="widefat">
                        <option value="0" <?php echo $options['use_google_autocomplete'] == 0 ? 'selected' : ''; ?>>
                            Jim's
                            Address Service
                        </option>
                        <option value="1" <?php echo $options['use_google_autocomplete'] == 1 ? 'selected' : ''; ?>>
                            Google
                        </option>
                    </select>


                    <div class="<?php echo is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden' ?>">
                        <!-- Google Address Validation -->
                        <label for="google-address-validation">Use Address Validation service</label>
                        <select name="google_address_validation" id="google-address-validation" class="widefat">
                            <option value="1" <?php echo $options['google_address_validation'] == 1 ? 'selected' : ''; ?>>
                                Enabled
                            </option>
                            <option value="0" <?php echo $options['google_address_validation'] == 0 ? 'selected' : ''; ?>>
                                Disabled
                            </option>
                        </select>
                    </div>

                    <!-- Google API Key -->
                    <label for="google-api-key">Google API Key</label>
                    <input type="password" class="widefat" id="google-api-key" name="google_api_key"
                           value="<?php echo isset($options['google_api_key']) ? $options['google_api_key'] : '' ?>">
                </div>

            </div>

            <div class="setting-group redirection">
                <br/><br/>
                <hr class="gradient-hr">
                <h3>Redirection Settings</h3>
                <div class="inner">
                    <label for="user-redirect">Set Redirect if you want to redirect after forms successful submission
                        (thank
                        you
                        page)</label>

                    <select name="use_redirect" id="use-redirect" class="widefat">
                        <option value="0" <?php echo $options['use_redirect'] == 0 ? 'selected' : ''; ?>>No</option>
                        <option value="1" <?php echo $options['use_redirect'] == 1 ? 'selected' : ''; ?>>Yes</option>
                    </select>


                    <label for="redirect-success-url">Set the URL you want the form to redirect after successful
                        submission</label>
                    <input type="text" class="widefat" id="redirect-success-url" name="redirect_success_url"
                           value="<?php echo isset($options['redirect_success_url']) ? $options['redirect_success_url'] : '' ?>">
                </div>
            </div>

            <div class="setting-group fms-communication-settings <?php echo is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden' ?>">
                <br/><br/>
                <hr class="gradient-hr">
                <h3>FMS Communication Settings</h3>
                <div class="inner">
                    <div class="settings-text">
                        <ul>
                            <li>These settings are used to communicate with the FMS API.</li>
                        </ul>
                    </div>

                    <label for="replace-nz-special">Replace Special Characters For New Zealand Leads</label>
                    <select name="replace_nz_special" id="replace-nz-special" class="widefat">
                        <option value="0" <?php echo $options['replace_nz_special'] == 0 ? 'selected' : ''; ?>>No
                        </option>
                        <option value="1" <?php echo $options['replace_nz_special'] == 1 ? 'selected' : ''; ?>>Yes
                        </option>
                    </select>
                </div>
            </div>

            <div class="setting-group target-environment <?php echo is_admin() && isset($_GET['debug_mode']) ? '' : 'hidden' ?>">
                <br/><br/>
                <hr class="gradient-hr">
                <h3>FMS Target Environment</h3>
                <div class="inner">
                    <div class="settings-text">
                        Please note that some of these environments are only available to Jim's Group staff and they
                        require
                        network authentication.
                    </div>
                    <select name="fms_environment" id="fms-environment" class="widefat">
                        <?php foreach ($fms_enviroment as $key => $value): ?>
                            <option value="<?php echo $key; ?>" <?php echo $options['fms_environment'] == $key ? 'selected' : ''; ?>><?php echo $value; ?></option>
                        <?php endforeach; ?>
                    </select>
                </div>

            </div>

            <div class="setting-group auto-update">
                <hr class="gradient-hr">
                <h3>Auto-Update Settings</h3>
                <div class="inner">
                    <div class="settings-text">The WPCB plugin offers two options for auto-updating:
                        <ul>
                            <li>Stable: This options installs the latest stable version of the plugin.</li>
                            <li>Early: This option installs the latest version of the plugin, including features that
                                are
                                still in development.
                            </li>
                        </ul>
                    </div>
                    <select name="auto_update" id="auto-update" class="widefat">
                        <option value="0" <?php echo $options['auto_update'] == 0 ? 'selected' : ''; ?>>Stable</option>
                        <option value="1" <?php echo $options['auto_update'] == 1 ? 'selected' : ''; ?>>Early</option>
                    </select>
                </div>
            </div>


            <div class="button-wrapper">
                <input id="save-form" class="button button-primary" type="submit" value="Save Values">

                <!-- Clear Form Button -->
                <!--                <input id="clear-form" type="reset" class="button button-warning" value="Clear Form">-->

                <!-- Delete Form Button -->
                <input id="delete-form" type="button" class="button button-danger" value="Delete Values">
            </div>
        </form>
    </section>

    <script>
        /** Add a shevron to each setting-group to allow the user to collapse the settings **/
        jQuery('.setting-group').each(function () {
            var $this = jQuery(this);
            var $header = $this.find('h3');
            var $chevron = jQuery('<i class="chevron chevron-down"></i>');
            $header.append($chevron);
            $header.click(function () {
                $this.find('.inner').toggleClass('hidden');
                $chevron.toggleClass('chevron-down');
                $chevron.toggleClass('chevron-up');
            });
        });
    </script>
</div>











