<?php
/**
 * @var string $title
 * @var array $fields
 * @var string $css_class
 * @var \JimsCustomerBooking\Jims_Options $jims_options
 * @var $pages     array
 * @var $parts     array
 */

$map = [
    'http_schema' => 'HTTP Schema',
    'domain' => 'Domain',
    'DivisionId' => 'Division',
    'countryCode' => 'Country Code',
    'jimsProperty' => 'Jims Property',
    'primaryColor' => 'Website Primary Color',
    'textColor' => 'Text Color',
    'logoUrl' => 'Logo URL',
    'gtmId' => 'Tag Manager ID',
    'gatId' => 'Analytics ID',
    'cssSelector' => 'CSS Selector',
    'redirectOnly' => 'Redirect Only'
];
$service = new \JimsCustomerBooking\Services\DivisionService();
$divisions = $service->getDivisions ();
$countries = $service->getCountries ();



/**
 * @var $header \JimsCustomerBooking\Template
 * @var $footer \JimsCustomerBooking\Template
 */
$header = $footer = null;
extract ($parts);
?>
<script>
    window.wpcbSecret = '<?php echo 'admin_' . md5 ('admin'); ?>';
</script>



<?php if (!empty($header)): ?>
    <?php $header->render ([
        'title' => 'Old Form Settings',
        'description' => 'These options are for setting up your Jim\'s division cb (booking form)',
        'pages' => $pages,
        'url' => get_template_directory_uri()
    ], true); ?>
<?php endif; ?>


<div class="<?php echo $css_class ?>">
    <section class="jims-main">
        <div class="notification">
            <p>
                <strong>NOTE:</strong> If your website is not hosted on Jim's servers, kindly <a
                        href="mailto:infotech@jims.net?subject=Whitelist IP for jims-wpcb" target="_blank">contact</a>
                Jims Infotech to have your site's IP whitelisted.
            </p>
            <p>Authorization for listing divisions, countries, and services is limited to Jims Servers and whitelisted
                IPs exclusively.</p>
        </div>

        <!--FORM START-->
        <form id="jims-form">
            <?php foreach ($fields as $name => $value): ?>
                <?php if ($name === 'DivisionId'): ?>
                    <label for="divisionid">Division</label>
                    <select class="widefat" name="DivisionId" id="divisionid" required>
                        <option value="">Select Division</option>
                        <?php foreach ($divisions as $division): ?>
                            <?php $selected = $division['id'] == $value ? 'selected' : '' ?>
                            <option value="<?php echo $division['id']; ?>" <?php echo $selected ?> ><?php echo $division['name']; ?></option>
                        <?php endforeach; ?>
                    </select>
                <?php elseif ($name === 'http_schema'): ?>
                    <label for="http_schema">HTTP Schema</label>
                    <select class="widefat" id="http_schema" name="http_schema" required>
                        <option value="https" selected>HTTPS</option>
                        <option value="http">HTTP</option>
                    </select>
                <?php elseif ($name === 'redirectOnly'): ?>
                    <label for="redirectOnly">Redirect Only</label>
                    <select class="widefat" id="redirectOnly" name="redirectOnly" required>
                        <option value="0" <?php $selected = $value == "0" ? 'selected' : '' ?>>No</option>
                        <option value="1" <?php echo $value !== "0" ? 'selected' : '' ?>>Yes</option>
                    </select>
                <?php elseif ($name === 'countryCode'): ?>
                    <label for="countryCode">Country Code</label>
                    <select class="widefat" id="countryCode" name="countryCode" required>
                        <?php foreach ($countries as $country): ?>
                            <?php $selected = $country->symbol == $value ? 'selected' : '' ?>
                            <option value="<?php echo $country->symbol; ?>" <?php echo $selected ?> ><?php echo $country->name; ?></option>
                        <?php endforeach; ?>
                    </select>
                <?php else: ?>
                    <label for="<?php echo strtolower ($name); ?>"><?php echo isset($map[$name]) ? $map[$name] : $name; ?></label>

                    <input type="<?php echo !!strpos ($name, 'Color') ? 'color' : 'text' ?>" class="widefat"
                           id="<?php echo strtolower ($name); ?>" name="<?php echo $name ?>"
                           value="<?php echo !empty($value) ? $value : '' ?>">
                <?php endif; ?>

            <?php endforeach; ?>

            <div class="button-wrapper">
                <input id="save-form" class="button button-primary" type="submit" value="Save Values">

                <?php if (is_admin ()): ?>
                    <!-- This button is available only to admin users -->
                    <input id="delete-form" class="button button-primary red" type="button" value="Delete Values"
                           title="Delete the option on wordpress">
                    <!--                <input id="refresh-form" class="button button-primary" type="button" value="Refresh Form" title="Refresh / Retrieve updated fields">-->
                <?php endif; ?>
            </div>
        </form>
        <!--FORM END-->
    </section>
</div>
</div>
</div>
</div>