<?php
/**
 * @var string $title
 * @var array $fields
 * @var array $services
 * @var string $css_class
 * @var \JimsCustomerBooking\Jims_Options $jims_options
 * @var $selected_division int
 * @var $pages     array
 * @var $parts     array
 * @var $native_fields array
 */

use JimsCustomerBooking\Helpers\StringHelper;

/**
 * @var $header \JimsCustomerBooking\Template
 * @var $footer \JimsCustomerBooking\Template
 */
$header = $footer = null;
extract($parts);
?>

<?php if (!empty($header)): ?>
    <?php $header->render([
        'title' => 'Shortcode Generator',
        'description' => 'This page allows to generate shortcodes for CB',
        'pages' => $pages,
        'url' => get_template_directory_uri()
    ], true); ?>
<?php endif; ?>

<div class="<?php echo $css_class ?>">

    <section class="cb3">
        <form name="native-booking-form-shortcode-generator">
            <input type="hidden" id="shortcode-name"
                   value="<?php echo StringHelper::GetPrefixed('booking_form', 'machine'); ?>">

            <!--- Radio Buttons -->
            <div class="form-group radios">
                <h2>Form Section Visibility Settings</h2>

                <div class="radio-group">
                    <!--- Popup Buttons -->
                    <wpcb-toggle class="use-popup" name="enable_popup" text="Use CB as popup"></wpcb-toggle>

                    <!-- Enable / Disable Country Field -->
                    <wpcb-toggle class="enable-country" name="show_country"
                                 text="Disable Country Selection"></wpcb-toggle>

                    <!-- Show / Hide Form Texts -->
                    <wpcb-toggle class="hide-info-texts" name="show_info_text"
                                 text="Hide fields info texts"></wpcb-toggle>

                    <!-- Use Contained / Headless Form -->
                    <wpcb-toggle class="hide-header" name="contained" text="Hide Header"></wpcb-toggle>
                </div>
            </div>

            <div class="form-group logo">
                <h2>Logo Settings</h2>
                <!-- Set Logo Image -->
                <label for="logo-image">Set Logo URL</label>
                <input type="url" name="logo_url" id="logo-url">

                <!-- Set Logo Visibility if contained isn't active -->
                <wpcb-toggle class="hide-logo" name="hide_logo" text="Hide Logo"></wpcb-toggle>
            </div>
            <p class="form-group__disabled-info">This section is available only if the header is visible.</p>

            <!--- Allow URL Params (only for admins), this enables extra fields-->
            <div class="form-group url-params">
                <h2>URL Params Settings</h2>
                <wpcb-notification>
                    URL Params features allows the form to be pre-filled with data from the URL. This is useful for
                    pre-filling the form with data from other systems.
                </wpcb-notification>
                <wpcb-toggle for="url-params" name="accept_url_params" class="allow-url-params"
                             text="Allow URL Params"></wpcb-toggle>
                <small>Available Url Params to include</small>
                <select class="url-params" id="url-params" name="allowed_url_params" multiple>
                    <option value="divisionId">Division Id</option>
                    <option value="countryCode">Country Code</option>
                </select>
            </div>

            <!--- Select steps  (only for admins) -->
            <div class="form-group steps">
                <h2>Form Steps</h2>
                <wpcb-notification>
                    This section allows to select the extra steps that will be displayed in the form. For safety reasons
                    the services and client info steps are always displayed.
                </wpcb-notification>
                <select class="steps" name="show_steps" multiple>
                    <option value="divisions">Divisions</option>
                </select>
            </div>

            <div class="form-group divisions">
                <h2>Division Settings</h2>
                <wpcb-notification>
                    This section allows to select the division that will be displayed in the form. If no division is
                    selected, the form will display all the divisions. By default, the selected division is the one set
                    on main settings page.
                </wpcb-notification>

                <label for="division-id">Select a different division for the shortcode</label>
                <select class="division-id" id="division-id" name="division_id">
                </select>

                <label for="show-divisions-only">Show only specific divisions on Division Select</label>
                <select class="show-divisions-only" id="show-divisions-only" name="show_only_divisions" multiple
                        disabled>
                </select>
            </div>

            <div class="form-group services">
                <h2>Service Settings</h2>
                <wpcb-notification>
                    This section allows to manage the services that will be displayed in the form. You can preselect
                    services, show only services or use a custom service list.
                </wpcb-notification>

                <!-- show services lists -->
                <div class="service-list">

                    <!-- Service Search -->
                    <div class="service-search-container">
                        <input type="text" name="service_search" id="service-search"
                               placeholder="Search for a service name...">
                        <img src="<?php echo plugins_url('/assets/dist/img/magnifying-glass.svg', JIMS_CBP_FILE); ?>">
                    </div>

                    <!-- Service Results -->
                    <div class="service-results empty"></div>

                    <wpcb-notification type="warning">
                        If you don't know the id of the service you are looking for, you can drag and drop the service
                        from the list above to the text field you want to add it.
                    </wpcb-notification>
                </div>

                <!-- Preselect Services -->
                <wpcb-toggle for="service-ids" class="preselect-services" text="Preselect Services"
                             name="preselect_services"></wpcb-toggle>
                <input type="text" name="service_ids" id="service-ids" value="" placeholder="Service Ids">
                <small>Comma separated list of service ids. This feature skips the service select step adding the
                    selected services to the final step.</small>

                <!-- Show Only Services -->
                <wpcb-toggle for="show-only-services" class="show-only-services"
                             text="Show Only Services"></wpcb-toggle>
                <input type="text" name="show_only_services" id="show-only-services" value="" placeholder="Service Ids">
                <small>Comma separated list of service ids. This feature hides all the other services on service select
                    and shows only those selected.</small>

                <!-- Use Custom Service List -->
                <wpcb-toggle for="service-list" class="use-custom-service-list" text="Use Custom Service List"
                             name="use_custom_service_list"></wpcb-toggle>
                <input type="text" name="service_list" id="service-list" value="" placeholder="Service List Name">
                <small>Use a custom service list. This feature uses an FMS Custom Service list. FMS Service Lists allows
                    you to show services that are not available on default list. This names can only be request from MCCC.</small>
            </div>

            <div class="form-group text-overrides">
                <h2>Text Overrides</h2>
                <wpcb-notification>
                    This section allows to override the default texts that are displayed in the form.
                </wpcb-notification>
                <!--- Text Overrides -->

                <!--                <label for="loader-search-text">Loader Search Text</label>-->
                <!--                <input type="text" name="loader_search_text" id="loader-search-text" value=""-->
                <!--                       placeholder="Searching for Franchisee in your Area...">-->
                <!--                <small>Replaces the text on the loader when searching for franchisees</small>-->

                <label for="submit-button-text">Submit Button Text</label>
                <input type="text" name="button_text" id="submit-button-text" value="" placeholder="Submit">
                <small>Replaces the text on the Submit button</small>
            </div>

            <div class="button-container">
                <button type="submit" class="button button-primary">Generate Shortcode</button>
            </div>
        </form>

        <p>To add the CB3 form shortcode just copy / paste the following:</p>
        <pre style="white-space: normal; word-break: break-all; outline: darkblue; background: darkcyan; color: white; border-radius: 10px; padding: 1em 0.5em;"
             id="native-booking-form-shortcode">
                <?php echo '[' . StringHelper::GetPrefixed('booking_form', 'machine') . ']' ?>
            </pre>
    </section>
</div>