<?php

namespace JimsGroup\Apis;

use JimsGroup\Utilities\HttpHeaders;
use JimsGroup\Utilities\HttpRequest;

//use JimsGroup\Utilities\Logger;

/**
 * Class AbstractApi
 * @package JimsGroup\Apis
 */
abstract class AbstractApi
{

    /**
     * Refers to the api urls keys
     * @var string
     */
    private $mode = 'production';


    /**
     * @var string[] $api_urls
     */
    protected $api_urls = [
        'production' => 'https://api.jims.net',
        'uat' => 'https://api.uat.aws.jims.net', //user acceptance testing
        'hf' => 'https://api.hf.aws.jims.net', // hotfix
        'int' => 'https://api.int.aws.jims.net/', //integration
        //'hf2' => 'https://api2.hf.aws.jims.net', // hotfix 2
        'development' => 'https://api.int.aws.jims.net',
        'local' => 'http://localhost:3000',
    ];

    private $response_headers = [];

    /**
     * @var array $paths
     */
    protected $paths = [];

    public function __construct($mode = 'production')
    {
        $this->mode = $mode;
    }

    /**
     * @param string $name
     * @return string
     */
    public function getEndpointUrl(string $name): string
    {
        return $this->getApiUrl() . $this->paths[$name];
    }

    /**
     * @return string
     */
    public function getApiUrl(): string
    {
        return $this->api_urls[$this->mode];
    }

    /**
     * @return string[] $api_urls
     */
    public function getApiUrls(): array
    {
        return $this->api_urls;
    }

    /**
     * @param mixed|string $mode
     */
    public function setMode(mixed $mode): void
    {
        $this->mode = $mode;
    }


    /**
     * @param $url string
     * @param $params array
     * @return string
     * @throws \JsonException
     */
    protected function post(string $url, array $params = [], array $extra_headers =[]): string
    {
        $request = new HttpRequest();
        $request->setUrl($url);
        $request->setMethod('POST');
        $request->setHeaders([
            'Content-Type: application/json',
            'Accept: application/json',
        ]);

        if(count($extra_headers) > 0){
            foreach ($extra_headers as $header) {
                $request->addHeader( $header );
            }
        }

        $request->setBody(json_encode($params, JSON_THROW_ON_ERROR));
        $request->send();

        $this->response_headers = $request->getResponseHeaders(true);

        return $request->getResponse();
    }

    /**
     * @param $url
     * @param array $params
     * @param array $extra_headers
     * @return string
     */
    protected function get(string $url, array $params = [], array $extra_headers = []): string
    {
        $request_headers = new HttpHeaders([
            'Content-Type' =>  'application/json',
            'Accept' => 'application/json',
        ]);

        if (count($extra_headers) > 0) {
            foreach ($extra_headers as $header_name => $header) {
                $request_headers->add($header_name, $header);
            }
        }

        if(count($params) > 0 && !empty($params)){
            $params_query = http_build_query($params);
            $url = $url . '?' . $params_query;
        }

        $request = new HttpRequest();
        $request->setUrl($url);
        $request->setMethod('GET');
        $request->setHeaders($request_headers->toCurlArray());
        $request->send();

        //uncomment to log headers
//        $headers = $request->getResponseHeaders(true);
//        error_log(json_encode($headers), 0);


        return $request->getResponse();
    }
}