<?php

namespace JimsGroup\Apis;

class FMSInfo extends AbstractApi implements IFmsApi, IApi
{
    /**
     * @var string[]
     */
    protected $paths = [
        'divisions' => '/api/divisions',
    ];

    /**
     * Authentication token, send in every request
     * as header basic auth
     *
     * @var string $header_auth_token
     */
    private $header_auth_token = '';

    private $payload = [];

    /**
     * Returns the list of divisions
     * by default it returns an array of objects
     *
     * @param bool|string $format, json|xml|false
     * @return mixed|string
     */
    public function getDivisions($format = false){
        $url = $this->getEndpointUrl('divisions');
        $result = [];

        if($format === 'xml'){
            $response = $this->get($url, [], [
                'Accept' =>  'application/xml'
            ]);
            $result = !empty($response) ? $response : $result;
        } else{
            $response = $this->get($url);

            //ensure the response for json format is a string and not array
            if(!empty($response) ) {
                $result = $format === 'json' ? $response : \json_decode($response, true);
            } else {
                $result = $format === 'json' ? '': $result;
            }
        }

        return $result;
    }

    /**
     * @param $countryCode
     * @return array|array[]
     */
    public function getDivisionIcons($countryCode = 'AU')
    {
        $customerBooking = new CustomerBooking();
        $customerBooking->setHeaderAuthToken($this->header_auth_token);

        try {
            $data = $customerBooking->getInitialBookingRequest([
                'countryCode' => $countryCode,
            ]);
        } catch (\Exception $e) {
            $data = [];
        }

        $result = [];
        if(count($data) > 3){
            foreach ($data as $service) {
                if(empty($service['group_id'])){
                    continue;
                }

                if(empty($result[$service['group_id']]['group_icon_url'])){
                    $result[$service['group_id']] = [
                        'icon' => $service['group_icon_url'],
                        'name' => $service['group_name'],
                        'id' => $service['group_id']
                    ];
                }
            }
        }

        return $result;
    }

    public function getHeaderAuthToken(): string
    {
        return $this->header_auth_token;
    }

    public function setHeaderAuthToken(string $header_auth_token): void
    {
        $this->header_auth_token = $header_auth_token;
    }
}