<?php

namespace JimsGroup\Datatypes;

/**
 * Class AllocationRequestType
 *
 * @method void addLeadNote($string, $clear = true)
 * @method void addMessageNote($string, $clear = true)
 */
class AllocationRequestType extends BookingRequestType
{
    protected $data = [];

    /**
     * @param array $data
     */
    public function __construct(array $data = [])
    {
        $this->data = array_merge( $this->default, $this->data, $data );
        $this->data['id'] = 0;
    }

    /**
     * @param $full_address
     * @param $geocode_y
     * @param $geocode_x
     * @param $street_number
     * @param $street_name
     * @param $suburb
     * @param $state
     * @param $country
     * @param $postcode
     * @return void
     */
    public function setBookingAddress($full_address, $geocode_y, $geocode_x, $street_number, $street_name, $suburb, $state, $country, $postcode, $unit_number = '')
    {
        $this->data['bookingAddress'] = new BookingAddress([
            'full_address' => $full_address,
            'geocode_y' => $geocode_y,
            'geocode_x' => $geocode_x,
            'geocode_accuracy' => 'ROOFTOP',
            'street_number' => $street_number,
            'street_name' => $street_name,
            'suburb' => $suburb,
            'state' => $state,
            'country' => $country,
            'postcode' => $postcode,
            'unit_number' => $unit_number
        ]);
    }

    /**
     * @param $address
     * @param $addressString
     * @param $allow_rating
     * @param $comments
     * @param $isUserInfoConfirmed
     * @param $emailAddress
     * @param $firstName
     * @param $lastName
     * @param $phoneNumber
     * @param $subscribed
     * @param $businessName
     * @return void
     */
    public function setUserInfo($address, $addressString, $allow_rating, $comments, $isUserInfoConfirmed, $emailAddress, $firstName, $lastName, $phoneNumber, $subscribed, $businessName)
    {
        $this->data['userInfo'] = new UserInfo([
            'address' => $address,
            'addressString' => $addressString,
            'allow_rating' => $allow_rating,
            'comments' => $comments,
            'isUserInfoConfirmed' => $isUserInfoConfirmed,
            'emailAddress' => $emailAddress,
            'firstName' => $firstName,
            'lastName' => $lastName,
            'phoneNumber' => $phoneNumber,
            'subscribed' => $subscribed,
            'businessName' => $businessName
        ]);
    }

    /**
     * The frontend will send the recaptcha response as a string
     *
     * @param $recaptchaResponse
     * @return void
     */
    public function setReCaptchaResponse($recaptchaResponse)
    {
        $this->data['reCaptchaResponse'] = $recaptchaResponse;
    }

    /**
     * @param $apiKey
     * @return void
     */
    public function setApiKey($apiKey)
    {
        $this->data['apiKey'] = $apiKey;
    }
}