<?php

namespace JimsGroup\Datatypes;

/**
 * Class BookingRequestType
 *
 * @package JimsGroup\Datatypes
 * List of all the properties that can be set in the BookingRequestType
 * @property int $pageIndex
 * @property int $progress
 * @property string $title
 * @property string $url
 * @property string $brandSource
 * @property bool $confirmed_not_previous_client
 * @property string $countryCode
 * @property string $jimsProperty
 * @property string $jimsProvider
 * @property string $otherServices
 * @property UserInfo|array $userInfo
 * @property string $utmCampaign
 * @property string $utmContent
 * @property string $utmMedium
 * @property string $utmSource
 * @property string $utmTerm
 * @property string $lead_note
 * @property string $message_note
 *
 */
abstract class BookingRequestType extends Datatype
{
    /**
     * @var array
     */
    protected $default = [
        'id' => 0,
        "addressId" => -1,
        'bookingStep' => null,
        'pageIndex' => 1,
        'progress' => 30,
        'title' => 'Select Service',
        'url' => 'servicesChoose',
        'brandSource' => 'Jims',
        'confirmed_not_previous_client' => true,
        'countryCode' => 'AU',
        'jimsProperty' => 'CB30',
        'jimsProvider' => 'Jims',
        'otherServices' => 'CB30_Book',
        'serviceListName' => '',
        'division_id' => 0,
        'bookingAddress' => null,
        'serviceLine' => null,
        'apiKey' => '',
        'selectedServicesIds' => [],
        'ipAddress' => '127.0.0.1',
        'userInfo' => array(),
        'utmCampaign' => '',
        'utmContent' => '',
        'utmMedium' => '',
        'utmSource' => '',
        'utmTerm' => '',
        'lead_note' => '',
        'message_note' => '',
    ];

    /**
     * @var array
     */
    protected $data = [];

    /**
     * @param $name
     * @return mixed
     */
    public function __get($name)
    {
        return $this->data[$name] ?? null;
    }

    /**
     * @param $name
     * @param $value
     * @return void
     */
    protected function set($name, $value)
    {
        $this->data[$name] = $value;
    }

    /**
     * @param array $data
     */
    protected function update($data)
    {
        $this->data = array_merge($this->data, $data);
    }

    /**
     * @param array $note
     * @param bool $clear
     * @return void
     */
    public function addLeadNote($note, $clear = true)
    {
        $this->data['lead_note'] = $clear ? $note : $this->data['lead_note'] . "\n" . $note;

        return;
    }


    /**
     * @param array $note
     * @param bool $clear
     * @return void
     */
    public function addMessageNote($note, $clear = true)
    {
        $this->data['message_note'] = $clear ? $note : $this->data['message_note'] . "\n" . $note;

        return;
    }

    /**
     * @return string
     */
    public function __toString()
    {
        return json_encode($this->data);
    }

    public function __call($name, $arguments)
    {
        if (strpos($name, 'has') === 0) {
            //find out the name, remove has, add an underscore before all capital letters and convert to lower case
            $name_with_underscores = strtolower(preg_replace('/(?<!^)[A-Z]/', '_$0', str_replace('has', '', $name)));

            // remove has and convert
            $name_with_capital = str_replace('has', '', $name);

            //check if the property exists
            return $this->has($name_with_underscores) || $this->has($name_with_capital);
        }

        return null;
    }


    /**
     * @param $name
     * @param bool $only_name
     * @return array|string|false
     */
    public function find($name, $only_name = false)
    {
        //find out the name, remove has, add an underscore before all capital letters and convert to lower case
        $name_variation_arr = [
            'with_underscores' => strtolower(preg_replace('/(?<!^)[A-Z]/', '_$0', $name)),
            'original' => $name,
        ];

        foreach ($name_variation_arr as $name_variation) {
            if ($this->has($name_variation)) {
                return $only_name ? $name_variation : [$name_variation => $this->data[$name_variation]];
            }
        }

        return false;
    }

    /**
     * @return array
     */
    public function toArray()
    {
        $data = $this->data;
        foreach ($data as $key => $value) {
            if ($value instanceof Datatype) {
                $data[$key] = $value->toArray();
            }
        }

        return $data;
    }

    public function validate(array $data)
    {
        return parent::validate($data); // TODO: Change the autogenerated stub
    }

    public function setCountryCode($countryCode)
    {
        $this->set('countryCode', $countryCode);
    }

    /**
     * @param $name
     * @return bool
     */
    protected function has($name)
    {
        return isset($this->data[$name]) && !empty($this->data[$name]);
    }
}