<?php

namespace JimsGroup\Datatypes;

class Collection
{
    public function __construct(array $items)
    {
        $this->items = $items;
    }

    public function count()
    {
        return count($this->items);
    }

    public function first()
    {
        return reset($this->items);
    }

    public function last()
    {
        return end($this->items);
    }

    public function find($key)
    {
        return $this->items[$key] ?? null;
    }

    /**
     * @param callable $callback
     * @return static
     */
    public function filter(callable $callback)
    {
        return new static(array_filter($this->items, $callback));
    }

    public function getItems()
    {
        return array_values ($this->items);
    }

    public function getItemsAsJson()
    {
        return json_encode($this->getItemsAsArray());
    }

    public function __toString()
    {
        return $this->getItemsAsJson();
    }

    public function toArray()
    {
        return $this->getItemsAsArray();
    }

    public function serialize()
    {
        return serialize($this->getItemsAsArray());
    }

    public function map(callable $callback)
    {
        return new static(array_map($callback, $this->items));
    }

    private function getItemsAsArray()
    {
        $items = [];
        foreach ($this->items as $item) {
            if ($item instanceof Arrayable) {
                $items[] = $item->toArray();
            } else {
                $items[] = $item;
            }
        }
        return $items;
    }
}