<?php

namespace JimsGroup\Datatypes;

abstract class Datatype
{
    protected $default = [];

    protected $data = [];

    /**
     * @var array of keys to ignore when validating
     */
    protected $ignore_keys = [
        'unit_number'
    ];

    public function __construct(array $data = [], $ignore_validation = false)
    {
        if (!$this->validate($data) && !$ignore_validation) {
            throw new \InvalidArgumentException('Invalid data');
        }

        $this->data = array_merge($this->default, $this->data, $data);
    }

    public function __get($name)
    {
        if (isset($this->data[$name])) {
            return $this->data[$name];
        }

        if(method_exists($this, $name)) {
            return $this->$name();
        }

        return null;
    }

    public function __set($name, $value)
    {
        $this->data[$name] = $value;
    }

    public function __isset($name)
    {
        return isset($this->data[$name]);
    }

    public function toArray()
    {
        return $this->data;
    }

    public function validate(array $data)
    {
        //if $ignore_keys is not empty, remove the keys from $data and $this->default
        if (!empty($this->ignore_keys)) {
           foreach ($this->ignore_keys as $key) {
               unset($data[$key]);
               unset($this->default[$key]);
           }
        }

        return count( array_diff_key ($data, $this->default) ) === 0;
    }
}