<?php

namespace JimsGroup\Facade;

/**
 * Class Authenticate
 */
class Authenticate
{

    /**
     * @var array $configurations
     */
    private array $configurations = [];


    /**
     * @param $username
     * @param $password
     * @param string $mode
     * @return array|null
     * @throws \JsonException
     */
    public function FmsLogin($username, $password, string $mode = 'production'): ?array
    {
        $type= 'fso';
        $fso_auth = $this->FmsFsoLogin($username, $password, $mode);

        if (!empty($fso_auth['info'])) {
            $user = $fso_auth;
        } else {
            $type= 'fse';
            $user = $this->FmsFseLogin($username, $password, $mode);
        }

        return [
            'type' => $type,
            'info' => $user['info'] ?? null,
        ];
    }

    /**
     * @param $username
     * @param $password
     * @param string $mode
     * @return array|null
     * @throws \JsonException
     */
    public function FmsFsoLogin($username, $password, string $mode = 'production'): ?array
    {
        if(!$this->hasFmsConfig()) return null;

        $auth = new \JimsGroup\Apis\FMSAuthentication('', $mode);

        $auth->setGrantType('password');
        $auth->setScope('openid profile roles jimsApi session');
        $auth->setToken($this->configurations['fms']['fso_token']);
        $auth->setMode($mode);

        return [
            'info' => $auth->getUserInfo($username, $password),
            'type' => 'fso'
        ];
    }


    /**
     * @param $username
     * @param $password
     * @param string $mode
     * @return array|null
     * @throws \JsonException
     */
    public function FmsFseLogin($username, $password, string $mode = 'production'): ?array
    {
        if(!$this->hasFmsConfig()) return null;

        $auth = new \JimsGroup\Apis\FMSAuthentication('', $mode);

        $auth->setGrantType('password');
        $auth->setScope('openid profile jimsjobsapi offline_access');
        $auth->setToken($this->configurations['fms']['fse_token']);
        $auth->setMode($mode);

        return [
            'info' => $auth->getUserInfo($username, $password),
            'type' => 'fse'
        ];
    }


    /**
     * @param string $mode
     * @return array
     */
    public function CustomerBookingAccessToken(string $mode = 'production'): array|null
    {
        //check if fms secret is set and if not throw an exception
        if(isset($this->configurations['fms']) && empty($this->configurations['fms']['secret']) ) {
            throw new \InvalidArgumentException('FMS configuration requires secret and token');
        }

        $auth = new \JimsGroup\Apis\FMSAuthentication($this->configurations['fms']['secret'], $mode);

        return $auth->RequestToken(
            'customer_booking',
            'client_credentials',
            'jimsApi'
        );
    }

    /**
     * @param array $configuration
     * @return void
     */
    public function setFmsConfig(array $configuration): void
    {
        //check if secret is set and not empty and token is set and not empty
        if (empty($configuration['secret']) || empty($configuration['fse_token']) || empty($configuration['fso_token'])) {
            throw new \InvalidArgumentException('FMS configuration requires secret and token');
        }

        $this->configurations['fms'] = $configuration;
    }

    /**
     * @return bool
     */
    public function hasFmsConfig(): bool
    {
        return isset($this->configurations['fms']);
    }
}